Package: scrutr
Title: Scrutinizing Collections of Structured Datasets
Version: 0.3.1
Authors@R: 
    person("Daniel", "Rakotomalala", , "rakdanielh@gmail.com", role = c("aut", "cre"))
Description: Provides a coherent interface for exploring and transforming
    multiple related data frames that share a common structure.
    Complements single-dataset inspection tools by operating across an
    entire collection at once. Also includes lightweight utilities for
    related file and folder management tasks.
License: MIT + file LICENSE
URL: https://github.com/danielrak/scrutr
BugReports: https://github.com/danielrak/scrutr/issues
Depends: R (>= 4.1.0)
Imports: dplyr, glue, lubridate, magrittr, purrr, rio, stats, stringr,
        tibble, tools, utils, writexl
Suggests: knitr, readxl, rmarkdown, spelling, testthat (>= 3.0.0)
VignetteBuilder: knitr, rmarkdown
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-24 19:44:12 UTC; rheri
Author: Daniel Rakotomalala [aut, cre]
Maintainer: Daniel Rakotomalala <rakdanielh@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-25 09:40:08 UTC
Built: R 4.5.2; ; 2026-05-01 19:04:57 UTC; unix

# Generated by roxygen2: do not edit by hand

export(chars_structure)
export(chars_structure_general)
export(convert_all)
export(convert_r)
export(detect_chars_structure)
export(detect_chars_structure_datasets)
export(dupl_show)
export(dupl_sources)
export(folder_structure_replicate)
export(inspect)
export(inspect_vars)
export(inspect_write)
export(ljoin_checks)
export(mask_convert_r)
export(mask_rename_r)
export(path_move)
export(rename_r)
export(replace_multiple)
export(table_prop)
export(vars_compclasses)
export(vars_compclasses_allsame)
export(vars_compclasses_not_allsame)
export(vars_detect)
export(vars_detect_everywhere)
export(vars_detect_not_everywhere)
importFrom(glue,glue)
importFrom(magrittr,"%>%")
importFrom(stats,nobs)

# scrutr

<!-- badges: start -->
[![CRAN status](https://www.r-pkg.org/badges/version/scrutr)](https://CRAN.R-project.org/package=scrutr)
[![CRAN downloads](https://cranlogs.r-pkg.org/badges/scrutr)](https://cran.r-project.org/package=scrutr)
[![R-CMD-check](https://github.com/danielrak/scrutr/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/danielrak/scrutr/actions/workflows/R-CMD-check.yaml)
[![Codecov test coverage](https://codecov.io/gh/danielrak/scrutr/branch/main/graph/badge.svg)](https://app.codecov.io/gh/danielrak/scrutr?branch=main)
<!-- badges: end -->

> `scrutr` was previously developed under the name `industtry` (2024-2025) and has been renamed ahead of its first CRAN release.

`scrutr` is an R toolkit for **scrutinizing collections of structured datasets** (data frames). It targets workflows where you need to apply the same inspection, profiling, or conversion procedure across many related tables.

Two guiding ideas:

1. **"Collections first"**: common tasks (inspecting, detecting schema differences, batch conversion) operate at the *collection* level, not one dataset at a time.
2. **"Operational tooling"**: lightweight helpers for paths, duplicates, join checks, and string replacements that recur in production data pipelines.

---

## Installation

``` r
# install.packages("devtools")
devtools::install_github("danielrak/scrutr")
```

---

## API overview

| Area | Main intent | Key functions |
|:---|:---|:---|
| Inspection & profiling | Profile one dataset or a whole folder; export diagnostics to Excel. | inspect(), inspect_write(), inspect_vars() |
| Schema detection | Detect variable presence across datasets; compare classes. | vars_detect(), vars_compclasses(), detect_chars_structure_datasets() |
| Batch conversion / renaming | Convert file formats or rename files at scale via Excel masks. | convert_r(), convert_all(), mask_convert_r(), mask_rename_r(), rename_r() |
| Data hygiene | Duplicate diagnostics, join checks, proportions. | dupl_show(), dupl_sources(), ljoin_checks(), table_prop() |
| Paths & filesystem | Replicate folder structures, move through paths. | folder_structure_replicate(), path_move() |
| Utilities | Batch string replacement. | replace_multiple() |

---

## Quick examples

### Inspect a single data frame

``` r
library(scrutr)
result <- inspect(CO2)
head(result)
```

### Inspect a folder of datasets and export to Excel

``` r
mydir <- file.path(tempdir(), "example")
dir.create(mydir, showWarnings = FALSE)

saveRDS(cars, file.path(mydir, "cars.rds"))
saveRDS(mtcars, file.path(mydir, "mtcars.rds"))

inspect_vars(
  input_path = mydir,
  output_path = mydir,
  output_label = "diagnostics",
  considered_extensions = "rds"
)
```

### Batch conversion with `convert_all()`

``` r
convert_all(
  input_folderpath = mydir,
  considered_extensions = "rds",
  to = "csv",
  output_folderpath = file.path(mydir, "csv_output")
)
```

### Mask-driven conversion with `convert_r()`

``` r
# 1. Generate an Excel mask template
mask_convert_r(output_path = mydir)

# 2. Fill in the mask, then run:
convert_r(
  mask_filepath = file.path(mydir, "mask_convert_r.xlsx"),
  output_path = mydir
)
```

# WARNING - Generated by {fusen} from dev/flat_rename_r.Rmd: do not edit by hand

#' Create an excel mask compatible with the rename_r() function. 
#' 
#' Create an excel mask compatible with the rename_r() function. This must be used on a collection of files, i.e stored within the same folder. 
#' 
#' @param input_path Character 1L. Folder containing the set of files to rename
#' @param output_filename Character 1L. File name of the excel mask.
#'
#' @return No return value, called for side effects. An Excel mask file
#'   is written to \code{input_path}.
#' 
#' @importFrom magrittr %>% 
#' @export
#' @examples
#' library(magrittr)
#' data(cars)
#' data(mtcars)
#'
#' mydir <- tempfile()
#' dir.create(mydir)
#'
#' saveRDS(cars, file.path(mydir, "cars.rds"))
#' saveRDS(mtcars, file.path(mydir, "mtcars.rds"))
#'
#' list.files(mydir)
#'
#' mask_rename_r(input_path = mydir)
#'
#' list.files(mydir)
#'
#' readxl::read_xlsx(file.path(mydir, "mask_rename_r.xlsx"))
#'
#' unlink(mydir, recursive = TRUE)
mask_rename_r <- function(input_path, 
                          output_filename = "mask_rename_r.xlsx"){
    
  if (! dir.exists(input_path)) {
    stop("input_path doesn't exist")
  }
  
  if (length(list.files(input_path)) < 2) {
    stop("input_path must contains at least two files. Renaming a single file with this function is not worth it.")
  }
  
  if (output_filename %in% list.files(input_path)) {
    
    stop(glue::glue("There is apparently already a mask in your folder: {output_filename}. Maybe you should complete it and run the rename_r() function."))
  }
  
  mask <- data.frame(file = list.files(input_path), 
                     renamed_file = NA, 
                     to_rename = NA)
  
  writexl::write_xlsx(
    mask, 
    path = file.path(input_path, output_filename))
}

# WARNING - Generated by {fusen} from dev/flat_rename_r.Rmd: do not edit by hand

test_that("Valid outputs are consistent", {
  
  library(magrittr)
  data(cars)
  data(mtcars)
  
  mydir <- tempfile()
  dir.create(mydir)
  
  saveRDS(cars, file.path(mydir, "cars.rds"))
  saveRDS(mtcars, file.path(mydir, "mtcars.rds"))
  
  expect_equal(list.files(mydir), 
               c("cars.rds", "mtcars.rds"))
  
  mask_rename_r(input_path = mydir)
  
  expect_equal(list.files(mydir), 
               c("cars.rds", "mask_rename_r.xlsx", "mtcars.rds"))
  
  expect_equal(
    readxl::read_xlsx(file.path(mydir, "mask_rename_r.xlsx")),
    structure(
      list(
        file = c("cars.rds", "mtcars.rds"),
        renamed_file = c(NA, NA),
        to_rename = c(NA, NA)
      ),
      class = c("tbl_df", "tbl", "data.frame"),
      row.names = c(NA, -2L)
    ))
  
  unlink(mydir, recursive = TRUE)
})

test_that("Errors are consistent", {
  
  library(magrittr)
  data(cars)
  data(mtcars)
  
  mydir <- tempfile()
  dir.create(mydir)
  
  saveRDS(cars, file.path(mydir, "cars.rds"))
  # no second save to test error
  
  expect_error(
    mask_rename_r(input_path = mydir %>% paste0("noexist")),
    regexp = "input_path doesn't exist")
  
  expect_error(
    mask_rename_r(input_path = mydir), 
    regexp = "input_path must contains at least two files. Renaming a single file with this function is not worth it.")
  
  file.create(file.path(mydir, "mask_rename_r.xlsx"))
  
  expect_error(
    mask_rename_r(input_path = mydir),
    regexp = "There is apparently already a mask in your folder: ")
  
  file.remove(file.path(mydir, "mask_rename_r.xlsx"))
})

# WARNING - Generated by {fusen} from dev/flat_rename_r.Rmd: do not edit by hand

test_that("Valid outputs are consistent", {
  
  library(magrittr)
  data(cars)
  data(mtcars)
  
  mydir <- tempfile()
  dir.create(mydir)
  
  # Two example files to rename: 
  saveRDS(cars, file.path(mydir, "cars.rds"))
  saveRDS(mtcars, file.path(mydir, "mtcars.rds"))
  
  # Create the mask: 
  mask_rename_r(input_path = mydir)
  
  # Fill the mask (in practice you can do it manually): 
  mask <- rio::import(file.path(mydir, "mask_rename_r.xlsx"))
  mask[["renamed_file"]] <- c("cars_renamed.rds", "mtcars_renamed.rds")
  mask[["to_rename"]] <- rep(1, 2)
  writexl::write_xlsx(mask, file.path(mydir, "mask_rename_r.xlsx"))
  
  result <- rename_r(mask_filepath = file.path(mydir, "mask_rename_r.xlsx"))
  
  expect_equal(result, list(TRUE, TRUE))
  
  expect_equal(
    list.files(mydir), 
    c("cars_renamed.rds", "mask_rename_r.xlsx", "mtcars_renamed.rds"))
  
  expect_equal(readRDS(file.path(mydir, "cars_renamed.rds")),
               cars)
  
  expect_equal(readRDS(file.path(mydir, "mtcars_renamed.rds")),
               mtcars)
  
  unlink(mydir, recursive = TRUE)
  })

test_that("Errors are consistent", {
  
  library(magrittr)
  data(cars)
  data(mtcars)
  
  mydir <- tempfile()
  dir.create(mydir)
  
  # Two example files to rename: 
  saveRDS(cars, file.path(mydir, "cars.rds"))
  saveRDS(mtcars, file.path(mydir, "mtcars.rds"))
  
  # Create the mask: 
  mask_rename_r(input_path = mydir)
  
  # Fill the mask (in practice you can do it manually): 
  mask <- rio::import(file.path(mydir, "mask_rename_r.xlsx"))
  mask[["renamed_file"]] <- c("cars_renamed.rds", "mtcars_renamed.rds")
  mask[["to_rename"]] <- rep(1, 2)
  
  mask <- dplyr::rename(mask, "to_rename2" = "to_rename")
  writexl::write_xlsx(mask, file.path(mydir, "mask_rename_r.xlsx"))
  
  expect_error(rename_r(file.path(mydir, paste0("mask_rename_r.xlsx", "noexist"))), 
               "mask_filepath doesn't exist. It must be a valid and full path including the xlsx file.")
  
  expect_error(rename_r(file.path(mydir, "mask_rename_r.xlsx")),
               "The mask must have the columns: file, renamed_file, to_rename")
  
  unlink(mydir, recursive = TRUE)
  })

# WARNING - Generated by {fusen} from dev/flat_dupl_show.Rmd: do not edit by hand

#' Show observations of all duplicated values of a variable or a combination of variables
#' @param data_frame Data.frame. Input data frame. Must be in the Global Environment and has a data.frame class
#' @param vars Character. Vector of variable or combination of variables
#' from which duplicates are checked
#' @return Data.frame. The part of inputted data frame with all observations of duplicated values of
#' indicated variable or combination of variables
#' @importFrom magrittr %>%
#' @export
#'
#' @examples
#' # A fictional data with duplicated values:
#' df <- data.frame("person_id" = c(1, 1, 2, 3,
#'                                  2, 4, 5, 5 ,1),
#'                  "person_age" = c(25, 25, 21, 32,
#'                                   21, 48, 50, 50, 52),
#'                  "survey_month" = c("jan", "feb", "mar", "apr",
#'                                     "apr", "may", "jun", "jul", "jan"),
#'                  "survey_answer" = c("no", "yes", "no", "yes",
#'                                      "yes", "yes", "no", "yes", NA))
#'
#' # Shuffling observations and columns to make duplicates difficult to see:
#' set.seed(1)
#' df <- df[sample(1:nrow(df)),
#'          sample(1:ncol(df))]
#' df
#'
#' # See all of the rows where person_id has more than an unique possible value: 
#' dupl_show(data = df, var = "person_id")
#'
#' # See all of the rows where the combination of person_id and survey_month variables has 
#' # more than an unique possible value : 
#' dupl_show(data = df, var = c("person_id", "survey_month"))
dupl_show <- function (data_frame, vars) {

  requireNamespace("magrittr")
  # Insert element-wise combination of vars input components within the data:
  # idvarsdupl
  data <- dplyr::mutate(data_frame,
                        idvarsdupl =
                          do.call(what = paste,
                                  purrr::map(vars, function (x)
                                    data_frame[[x]])))

  # Filter rows with values of idvarsdupl that appears
  # at least for the second time:
  data %>% dplyr::filter(duplicated(idvarsdupl)) %>%

    # extract idvarsdupl variable from the filtered data:
    dplyr::pull(idvarsdupl) %>%

    # extract all corresponding observations from data input:
    (function (p) dplyr::filter(data, idvarsdupl %in% p)) %>%

    # arrange values to visualize the duplicated cases:
    dplyr::arrange(!!dplyr::sym("idvarsdupl")) %>%

    # put vars input to first variable(s) location(s):
    dplyr::relocate(dplyr::all_of(vars)) %>%

    # remove idvarsdupl:
    dplyr::select(- !!dplyr::sym("idvarsdupl"))
}

test_that("dupl_show finds duplicates on single key", {
  df <- data.frame(
    person_id = c(1, 1, 2, 3, 2, 4, 5, 5, 1),
    person_age = c(25, 25, 21, 32, 21, 48, 50, 50, 52),
    survey_month = c("jan", "feb", "mar", "apr", "apr", "may", "jun", "jul", "jan"),
    survey_answer = c("no", "yes", "no", "yes", "yes", "yes", "no", "yes", NA)
  )

  set.seed(1)
  df <- df[sample(1:nrow(df)), sample(1:ncol(df))]

  result <- dupl_show(df, "person_id")

  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 7)
  expect_equal(names(result)[1], "person_id")
  expect_true(all(result$person_id %in% c(1, 2, 5)))
})

test_that("dupl_show finds duplicates on compound key", {
  df <- data.frame(
    person_id = c(1, 1, 2, 3, 2, 4, 5, 5, 1),
    person_age = c(25, 25, 21, 32, 21, 48, 50, 50, 52),
    survey_month = c("jan", "feb", "mar", "apr", "apr", "may", "jun", "jul", "jan"),
    survey_answer = c("no", "yes", "no", "yes", "yes", "yes", "no", "yes", NA)
  )

  set.seed(1)
  df <- df[sample(1:nrow(df)), sample(1:ncol(df))]

  result <- dupl_show(df, c("person_id", "survey_month"))

  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 2)
  expect_equal(names(result)[1], "person_id")
  expect_equal(names(result)[2], "survey_month")
})

test_that("dupl_show returns empty for no duplicates", {
  df <- data.frame(id = 1:5, val = letters[1:5])
  result <- dupl_show(df, "id")
  expect_equal(nrow(result), 0)
})

#' Variable class comparison - inconsistent types across datasets
#'
#' @param vars_compclasses_table Output of the \code{vars_compclasses()} function.
#' @return A subset of \code{vars_compclasses_table} containing only variables
#'   with different classes across datasets where they appear.
#' @examples
#' data_list <- list(cars = cars, mtcars = mtcars)
#' vcompclasses <- vars_compclasses(data_list)
#' vars_compclasses_not_allsame(vcompclasses)
#' @importFrom magrittr %>%
#' @export
#'
vars_compclasses_not_allsame <- function(vars_compclasses_table) {

  not_allsame <- apply(vars_compclasses_table, 1, function(x) {
    classes <- unique(x[-1])
    classes <- classes[classes != "-"]
    length(classes) != 1
  }) %>% unlist

  vars_compclasses_table[not_allsame, ]
}

test_that("inspect_vars creates correct Excel output", {
  mydir <- file.path(tempdir(), "test_inspect_vars")
  dir.create(mydir, showWarnings = FALSE)
  on.exit(unlink(mydir, recursive = TRUE))

  saveRDS(cars, file.path(mydir, "cars1.rds"))
  saveRDS(mtcars, file.path(mydir, "cars2.rds"))

  result_path <- inspect_vars(
    input_path = mydir,
    output_path = mydir,
    output_label = "cardata",
    considered_extensions = "rds"
  )

  expect_true(file.exists(file.path(mydir, "inspect_vars_cardata.xlsx")))

  # Read all 10 sheets
  sheets <- purrr::map(1:10, \(x)
    rio::import(file.path(mydir, "inspect_vars_cardata.xlsx"), sheet = x))

  # Sheet 1: dims
  dims <- sheets[[1]]
  expect_equal(dims$datasets, c("cars1", "cars2"))
  expect_equal(dims$nobs, c(50, 32))
  expect_equal(dims$nvar, c(2, 11))

  # Sheet 2: inspect_tot
  inspect_tot <- sheets[[2]]
  expect_true("datasets" %in% names(inspect_tot))
  expect_true("variables" %in% names(inspect_tot))
  expect_equal(nrow(inspect_tot), 13)  # 2 + 11 variables
  expect_true(all(c("cars1", "cars2") %in% inspect_tot$datasets))

  # Sheet 3: individual inspection for cars1
  inspect_cars1 <- sheets[[3]]
  expect_equal(inspect_cars1[[1]][1], "Obs =")  # First row is metadata

  # Sheet 5: vars_detect (columns should be clean names without extension)
  vars_detect_sheet <- sheets[[5]]
  expect_true("vars_union" %in% names(vars_detect_sheet))
  expect_true("cars1" %in% names(vars_detect_sheet))
  expect_true("cars2" %in% names(vars_detect_sheet))
  expect_equal(nrow(vars_detect_sheet), 13)

  # Sheet 6: vars_detect_everywhere (no common variables)
  expect_equal(nrow(sheets[[6]]), 0)

  # Sheet 7: vars_detect_not_everywhere (all 13)
  expect_equal(nrow(sheets[[7]]), 13)

  # Sheet 8: vars_compclasses
  expect_true("vars_union" %in% names(sheets[[8]]))

  # Sheet 9: vars_compclasses_allsame (all numeric -> all "same")
  expect_equal(nrow(sheets[[9]]), 13)

  # Sheet 10: vars_compclasses_not_allsame (none)
  expect_equal(nrow(sheets[[10]]), 0)
})

test_that("inspect_vars handles multiple extensions", {
  mydir <- file.path(tempdir(), "test_inspect_vars_multi")
  dir.create(mydir, showWarnings = FALSE)
  on.exit(unlink(mydir, recursive = TRUE))

  saveRDS(cars, file.path(mydir, "data1.rds"))
  rio::export(mtcars, file.path(mydir, "data2.csv"))

  # Only import rds files
  inspect_vars(
    input_path = mydir,
    output_path = mydir,
    output_label = "test",
    considered_extensions = "rds"
  )

  xlsx_path <- file.path(mydir, "inspect_vars_test.xlsx")
  expect_true(file.exists(xlsx_path))
  dims <- rio::import(xlsx_path, sheet = 1)
  expect_equal(nrow(dims), 1)  # Only one rds file
})

test_that("inspect_vars errors on empty folder", {
  mydir <- file.path(tempdir(), "test_inspect_vars_empty")
  dir.create(mydir, showWarnings = FALSE)
  on.exit(unlink(mydir, recursive = TRUE))

  expect_error(
    inspect_vars(input_path = mydir, output_path = mydir,
                 output_label = "test", considered_extensions = "rds"),
    "No files found"
  )
})

test_that("inspect_vars does not pollute global environment", {
  mydir <- file.path(tempdir(), "test_inspect_vars_noenv")
  dir.create(mydir, showWarnings = FALSE)
  on.exit(unlink(mydir, recursive = TRUE))

  saveRDS(cars, file.path(mydir, "mydata1.rds"))
  saveRDS(mtcars, file.path(mydir, "mydata2.rds"))

  before <- ls(envir = globalenv())

  inspect_vars(
    input_path = mydir,
    output_path = mydir,
    output_label = "noenv",
    considered_extensions = "rds"
  )

  after <- ls(envir = globalenv())

  # No new objects should appear in global env
  expect_equal(length(setdiff(after, before)), 0)
})

# WARNING - Generated by {fusen} from dev/flat_detect_chars_structure_datasets.Rmd: do not edit by hand

#' Detect character structure from datasets
#'
#' @param datasets_folderpath Character 1L. Folder path of datasets to process. These datasets must be at the root of the path
#' @param considered_extensions Character. Datasets file extensions to consider. Extensions must be one supported by the rio:: package
#' @param patterns Character. Patterns to detect across the datasets variables. Regex is supported
#' @param output_filepath Character 1L. Output folder path. 
#' @param get_output_in_session Logical 1L. If TRUE, the function return a list, such that each element element corresponds to pattern detection details for each considered dataset
#' @return If \code{get_output_in_session} is \code{TRUE}, a named list of data
#'   frames (one per dataset file), each with columns \code{var} (variable name),
#'   \code{any_defined_structure} (logical), and \code{examples} (character).
#'   The list is also saved as an RDS file at \code{output_filepath}.
#'   If \code{get_output_in_session} is \code{FALSE}, the function returns
#'   \code{NULL} invisibly and is called for its side effect of writing the RDS file.
#' @export
#'
#' @examples
#' mydir <- system.file("detect_chars_structure_datasets", package = "scrutr")
#' outfile <- file.path(tempdir(), "detect_college.rds")
#'
#' detect <- detect_chars_structure_datasets(
#'   datasets_folderpath = mydir,
#'   considered_extensions = "xlsx",
#'   patterns = "(?i)college",
#'   output_filepath = outfile,
#'   get_output_in_session = TRUE)
#'
#' # head(lapply(detect, head))
#'
#' file.exists(outfile)
#' unlink(outfile)
detect_chars_structure_datasets <- function (
  datasets_folderpath, 
  considered_extensions, 
  patterns, 
  output_filepath = file.path(
    datasets_folderpath, 
    paste0("detect_chars_structure_", basename(datasets_folderpath), ".rds")), 
  get_output_in_session = TRUE) {
  
  if (isFALSE(dir.exists(datasets_folderpath))) {
    stop("datasets_folderpath must be a valid folder path")
  }
  
  if (isFALSE(dir.exists(dirname(output_filepath)))) {
    stop("Folder path of output_filepath must be a valid one")
  }
  
  ext <- paste0(paste0("\\.", considered_extensions, "$"), collapse = "|")
  lfiles <- list.files(datasets_folderpath, full.names = TRUE)
  lfiles <- lfiles[grepl(ext, lfiles)]
  
  data_list <- lapply(lfiles, \(x) {

    dataname <- basename(x)
    data <- rio::import(x, trust = TRUE)
    # Coerce all columns to character so detect_chars_structure (which
    # requires character input) can be applied to every variable.
    data <- dplyr::mutate(data, dplyr::across(
      dplyr::everything(), as.character))
    data
  })
  
  det_structure_list <- lapply(
    data_list, \(each_data) 
    lapply(each_data, detect_chars_structure, 
           patterns = patterns, verbose = TRUE))
  
  det_structure_list2 <- lapply(det_structure_list, \(each_data) {
    
    tobind <- lapply(names(each_data), \(each_variable) {
      
      c("var" = each_variable, 
        "any_defined_structure" = each_data[[each_variable]][[1]], 
        "examples" = paste0(each_data[[each_variable]][[2]][
          1:min(10, length(each_data[[each_variable]][[2]]))], 
          collapse = " / "))
    })
    
    df_out <- as.data.frame(do.call(what = rbind, tobind))
    df_out[["any_defined_structure"]] <- as.logical(df_out[["any_defined_structure"]])
    df_out
  })
  
  det_structure_list2 <- stats::setNames(det_structure_list2, basename(lfiles))
  
  saveRDS(det_structure_list2, file = output_filepath)
  if (get_output_in_session) {det_structure_list2}}

# WARNING - Generated by {fusen} from dev/flat_detect_chars_structure_datasets.Rmd: do not edit by hand

test_that("Valid outputs are consistent", {

  mydir <- system.file("detect_chars_structure_datasets", package = "scrutr")
  outfile <- file.path(tempdir(), "detect_college.rds")

  detect <- detect_chars_structure_datasets(
  datasets_folderpath = mydir,
  considered_extensions = "xlsx",
  patterns = "(?i)college",
  output_filepath = outfile,
  get_output_in_session = TRUE)

  expect_true(file.exists(outfile))
  on.exit(unlink(outfile), add = TRUE)

  expect_equal(head(lapply(detect, head)),
               list(AK.xlsx = structure(list(var = c("School Code", "School Name",
                                                     "Address", "City", "State Code", "Zip Code"), any_defined_structure = c(FALSE,
                                                                                                                             TRUE, FALSE, FALSE, FALSE, FALSE), examples = c("NA", "ALASKA BIBLE COLLEGE / ALASKA CAREER COLLEGE / CHARTER COLLEGE / ILISAGVIK COLLEGE / ALASKA CHRISTIAN COLLEGE",
                                                                                                                                                                             "NA", "NA", "NA", "NA")), row.names = c(NA, 6L), class = "data.frame"),
                    AL.xlsx = structure(list(var = c("School Code", "School Name",
                                                     "Address", "City", "State Code", "Zip Code"), any_defined_structure = c(FALSE,
                                                                                                                             TRUE, TRUE, FALSE, FALSE, FALSE), examples = c("NA", "EDWARD VIA VIRGINIA COLLEGE OF OSTEOPATH / CENTRAL ALABAMA COMMUNITY COLLEGE / BIRMINGHAM SOUTHERN COLLEGE / CALHOUN COMMUNITY COLLEGE / ENTERPRISE STATE COMMUNITY COLLEGE / GADSDEN STATE COMMUNITY COLLEGE / GEORGE C WALLACE COMMUNITY COLLEGE / HUNTINGDON COLLEGE / COASTAL ALABAMA COMMUNITY COLLEGE - BREW / JEFFERSON STATE COMMUNITY COLLEGE",
                                                                                                                                                                            "5735 COLLEGE PARKWAY / 2602 COLLEGE DRIVE", "NA", "NA",
                                                                                                                                                                            "NA")), row.names = c(NA, 6L), class = "data.frame"), AR.xlsx = structure(list(
                                                                                                                                                                              var = c("School Code", "School Name", "Address", "City",
                                                                                                                                                                                      "State Code", "Zip Code"), any_defined_structure = c(FALSE,
                                                                                                                                                                                                                                           TRUE, TRUE, FALSE, FALSE, FALSE), examples = c("NA",
                                                                                                                                                                                                                                                                                          "ARKANSAS BAPTIST COLLEGE / LYON COLLEGE / CENTRAL BAPTIST COLLEGE / CROWLEYS RIDGE COLLEGE / HENDRIX COLLEGE / PHILANDER SMITH COLLEGE / PHILLIPS COUNTY COMMUNITY COLLEGE / SHORTER COLLEGE / U OF A COMM COLLEGE AT MORRILTON / UNIVERSITY OF ARKANSAS COMMUNITY COLLEGE",
                                                                                                                                                                                                                                                                                          "1501 COLLEGE AVENUE / 415 COLLEGE AVENUE / 100 COLLEGE DRIVE / ONE COLLEGE CIRCLE / 101 COLLEGE DR / 183 COLLEGE DRIVE / 1100 COLLEGE DRIVE / 218 COLLEGE DRIVE / ONE COLLEGE DRIVE / 1600 SOUTH COLLEGE STREET",
                                                                                                                                                                                                                                                                                          "NA", "NA", "NA")), row.names = c(NA, 6L), class = "data.frame"),
                    AS.xlsx = structure(list(var = c("School Code", "School Name",
                                                     "Address", "City", "State Code", "Zip Code"), any_defined_structure = c(FALSE,
                                                                                                                             TRUE, FALSE, FALSE, FALSE, FALSE), examples = c("NA", "AMERICAN SAMOA CMNTY COLLEGE",
                                                                                                                                                                             "NA", "NA", "NA", "NA")), row.names = c(NA, 6L), class = "data.frame"),
                    AZ.xlsx = structure(list(var = c("School Code", "School Name",
                                                     "Address", "City", "State Code", "Zip Code"), any_defined_structure = c(FALSE,
                                                                                                                             TRUE, FALSE, FALSE, FALSE, FALSE), examples = c("NA", "BROOKLINE COLLEGE - MESA / BROOKLINE COLLEGE - TUCSON / ARIZONA WESTERN COLLEGE / COCHISE COLLEGE / EASTERN ARIZONA COLLEGE / GLENDALE COMMUNITY COLLEGE / MESA COMMUNITY COLLEGE / PHOENIX COLLEGE / YAVAPAI COLLEGE / PIMA COUNTY COMMUNITY COLLEGE",
                                                                                                                                                                             "NA", "NA", "NA", "NA")), row.names = c(NA, 6L), class = "data.frame"),
                    CA.xlsx = structure(list(var = c("School Code", "School Name",
                                                     "Address", "City", "State Code", "Zip Code"), any_defined_structure = c(FALSE,
                                                                                                                             TRUE, TRUE, FALSE, FALSE, FALSE), examples = c("NA", "PLATT COLLEGE / PLATT COLLEGE - ANAHEIM / BRIGHTWOOD COLLEGE - CHULA VISTA / UNITED EDUCATION INSTITUTE - UEI COLLEGE / UEI COLLEGE-RIVERSIDE / PLATT COLLEGE - RIVERSIDE / UEI COLLEGE - BAKERSFIELD / UEI COLLEGE - SACRAMENTO / UC-HASTINGS COLLEGE OF THE LAW / ALLAN HANCOCK COLLEGE",
                                                                                                                                                                            "800 SO COLLEGE DRIVE / 550 N COLLEGE AVE / 835 COLLEGE AVENUE / 800 COLLEGE AVE / 8800 GROSSMONT COLLEGE DR / 4700 COLLEGE OAK DR / 435 COLLEGE AVE / ONE COLLEGE DRIVE / 100 EAST COLLEGE AVE / 7250 MESA COLLEGE DRIVE",
                                                                                                                                                                            "NA", "NA", "NA")), row.names = c(NA, 6L), class = "data.frame")))
})

test_that("Errors are consistent", {
  
  mydir <- system.file("detect_chars_structure_datasets", package = "scrutr")
  
  expect_error(detect_chars_structure_datasets(datasets_folderpath = "noexist"),
               regexp = "datasets_folderpath must be a valid folder path")
  
  expect_error(detect_chars_structure_datasets(
    datasets_folderpath = mydir, 
    considered_extensions = "xlsx", 
    patterns = "(?i)college", 
    output_filepath = "noexist/noexist.ext",
    get_output_in_session = FALSE),
    regexp = "Folder path of output_filepath must be a valid one")
})

# WARNING - Generated by {fusen} from dev/flat_detect_chars_structure.Rmd: do not edit by hand

#' Detect character structure
#' Detect if values within a character variable match at least one of defined patterns
#'
#' @param vector Character. Input vector to detect pattern from
#' @param patterns Character. Patterns to detect within vector. Regex is supported
#' @param verbose Logical 1L. If TRUE, additional details related to the pattern detection are provided
#' @return Logical 1L. If verbose is set to TRUE, the function returns a list with the following elements in order: 
#'    - "Any defined structure" : Logical 1L. TRUE if the pattern is detected anywhere from the input vector
#'    - "Which" : Character. Unique values of input vector matching the defined patterns
#'    - "Where" : Integer. Indexes of values from input vector matching the defined patterns
#' @export
#'
#' @examples
#' detect_chars_structure(
#'   vector = c("ABCD1234", "4567EF", "89GHIJ10"), 
#'   patterns = "[:alpha:]{4}" # detect four consecutive alphabetic values
#' )
#'
#' detect_chars_structure(
#'   vector = c("ABCD1234", "4567EF", "89GHIJ10"), 
#'   patterns = "[:alpha:]{4}", 
#'   verbose = TRUE
#' )
detect_chars_structure <- function (vector, patterns, verbose = FALSE) {
  
  if (isFALSE(is.character(vector))) {
    stop("vector must be of type character")
  }
  
  uniques <- stats::na.omit(unique(as.character(vector)))
  detect <- stringr::str_detect(uniques, paste0(patterns, collapse = "|"))
  check <- isTRUE(any(detect))
  
  if (isFALSE(verbose)) {
    check
  } else {
    
    which <- unique(uniques[detect])
    where <- which(vector %in% which)
    
    list("Any defined structure" = check, 
         "Which" = which, 
         "Where" = where)
  }
}

# WARNING - Generated by {fusen} from dev/flat_detect_chars_structure.Rmd: do not edit by hand

test_that("Valid outputs are consistent", {
  expect_true(detect_chars_structure(
  vector = c("ABCD1234", "4567EF", "89GHIJ10"), 
  patterns = "[:alpha:]{4}"))
  
  expect_equal(
    detect_chars_structure(
      vector = c("ABCD1234", "4567EF", "89GHIJ10"), 
      patterns = "[:alpha:]{4}", 
    verbose = TRUE),
    list(`Any defined structure` = TRUE, Which = c("ABCD1234", "89GHIJ10"
    ), Where = c(1L, 3L))
  )})

test_that("Errors are consistent", {
  
  expect_error(
    detect_chars_structure(
      vector = c(1:4), 
      patterns = "[:digit:]{1}"),
    regexp = "vector must be of type character")
})

#' Move through paths
#'
#' @param path_vector Character. Vector of paths with equal number of levels.
#' @param path_separator Character. Path separator
#'   (adapted to your OS, e.g. \code{"/"}).
#' @param move Integer. If positive, outputs path up to the specified level.
#'   If negative, removes the last specified level(s).
#'
#' @return Character vector of transformed paths.
#' @importFrom magrittr %>%
#' @importFrom glue glue
#' @export
#'
#' @examples
#' pvector <- c(
#'   "level_1/level_2/level_3/file_1.ext",
#'   "level_1/level_2/level_3/file_2.ext"
#' )
#'
#' path_move(path_vector = pvector,
#'           path_separator = "/",
#'           move = 1)
#'
#' path_move(path_vector = pvector,
#'           path_separator = "/",
#'           move = 2)
#'
#' path_move(path_vector = pvector,
#'           path_separator = "/",
#'           move = -1)
#'
#' path_move(path_vector = pvector,
#'           path_separator = "/",
#'           move = -2)
path_move <- function(path_vector, path_separator = "/", move) {

  if (!is.character(path_vector)) {
    stop("path_vector must be a character vector")
  }

  if (!is.numeric(move)) {
    stop("move must be a numeric")
  }

  if (move == 0) {
    stop("move must be different from 0")
  }

  split <- strsplit(path_vector, path_separator, fixed = TRUE)
  plevels <- purrr::map_dbl(split, length)

  if (length(rle(plevels)$values) != 1) {
    df_error <- data.frame(path_vector = path_vector,
                           number_of_levels = plevels)
    message(paste(utils::capture.output(print(df_error)), collapse = "\n"))
    stop("All path_vector elements must have the same number of levels")
  }

  pmatrix <- split %>% do.call(what = rbind)
  ncol <- ncol(pmatrix)
  move <- as.integer(move)

  if (move > 0) {
    if (!move %in% 1:ncol) {
      stop(glue::glue("level must be comprised between one of these values : {
                      paste(1:ncol, collapse = ', ')}"))
    }

    pmatrix[, 1:move, drop = FALSE] %>%
      apply(1, \(x) paste(x, collapse = path_separator))

  } else {
    if (abs(move) >= ncol(pmatrix)) {
      stop(glue::glue(
        "move absolute value must be strictly positive and strictly lesser than {ncol}"))
    }

    pmatrix[, -(ncol(pmatrix):(ncol(pmatrix) + move + 1)), drop = FALSE] %>%
      apply(1, \(x) paste(x, collapse = path_separator))
  }
}

test_that("path_move forward works", {
  pvector <- c(
    "level_1/level_2/level_3/file_1.ext",
    "level_1/level_2/level_3/file_2.ext"
  )

  expect_identical(path_move(pvector, "/", 1), c("level_1", "level_1"))
  expect_identical(path_move(pvector, "/", 2), c("level_1/level_2", "level_1/level_2"))
  expect_identical(path_move(pvector, "/", 3),
                   c("level_1/level_2/level_3", "level_1/level_2/level_3"))
})

test_that("path_move backward works", {
  pvector <- c(
    "level_1/level_2/level_3/file_1.ext",
    "level_1/level_2/level_3/file_2.ext"
  )

  expect_identical(path_move(pvector, "/", -1),
                   c("level_1/level_2/level_3", "level_1/level_2/level_3"))
  expect_identical(path_move(pvector, "/", -2),
                   c("level_1/level_2", "level_1/level_2"))
  expect_identical(path_move(pvector, "/", -3), c("level_1", "level_1"))
})

test_that("path_move with custom separator", {
  pvector <- c("a\\b\\c", "d\\e\\f")
  expect_identical(path_move(pvector, "\\", 2), c("a\\b", "d\\e"))
})

test_that("path_move errors correctly", {
  pvector <- c(
    "level_1/level_2/level_3/file_1.ext",
    "level_1/level_2/level_3/file_2.ext"
  )

  expect_error(path_move(pvector, "/", 0), "move must be different from 0")
  expect_error(path_move(pvector, "/", 5),
               "level must be comprised between one of these values")
  expect_error(path_move(pvector, "/", -4),
               "move absolute value must be strictly positive")
  expect_error(path_move(123, "/", 1), "path_vector must be a character vector")
  expect_error(path_move(pvector, "/", "a"), "move must be a numeric")

  # Unequal levels
  unequal <- c("a/b/c", "d/e")
  expect_error(path_move(unequal, "/", 1),
               "All path_vector elements must have the same number of levels")
})

# WARNING - Generated by {fusen} from dev/flat_utils.Rmd: do not edit by hand

#' Perform a classical dplyr::left_join() and add check information related to join
#'
#' @param ltable Data.frame. Left data frame in the join
#' @param rtable Data.frame. Right data frame in the join
#' @param ... Any other arguments passed to \code{dplyr::left_join()}.
#' @return Data.frame. Output of dplyr::left_join() with messages on number of observations
#' in left, right and joined data frames and list of common variables between
#' ltable and rtable
#' @importFrom magrittr  %>%
#' @export
#'
#' @examples
#' left_table <- data.frame("person_id" = c(1, 1, 2, 3,
#'                                  2, 4, 5, 5 ,1),
#'                  "person_age" = c(25, 25, 21, 32,
#'                                   21, 48, 50, 50, 52),
#'                  "survey_month" = c("jan", "feb", "mar", "apr",
#'                                     "apr", "may", "jun", "jul", "jan"),
#'                  "survey_answer" = c("no", "yes", "no", "yes",
#'                                      "yes", "yes", "no", "yes", NA))
#'
#' right_table <- data.frame("person_id" = c(2, 5, 4, 3, 1), 
#'                           "person_name" = c("John", "Marie", "Pierre", "Marc", "Jimmy"))
#'
#' list("left_table" = left_table, 
#'      "right_table" = right_table)
#'
#' ljoin_checks(left_table, right_table, by = "person_id")
ljoin_checks <- function (ltable, rtable, ...) {

  requireNamespace("magrittr")

  result <- dplyr::left_join(x = ltable, y = rtable, ...)

  lrows <- nrow(ltable)
  rrows <- nrow(rtable)
  jrows <- nrow(result)

  lvars <- names(ltable)
  rvars <- names(rtable)
  ivars <- intersect(lvars, rvars)

  message("Checks : \n",
          paste0("ltable rows : ", lrows),
          "\n",
          paste0("rtable rows :", rrows),
          "\n",
          paste0("jtable rows : ", jrows),
          "\n",
          paste0(ivars, collapse = ", ") %>%
            paste0(" are common var names accross the two tables"))

  return(result)
}

test_that("ljoin_checks performs correct join", {
  left_table <- data.frame(
    person_id = c(1, 1, 2, 3, 2, 4, 5, 5, 1),
    person_age = c(25, 25, 21, 32, 21, 48, 50, 50, 52)
  )
  right_table <- data.frame(
    person_id = c(2, 5, 4, 3, 1),
    person_name = c("John", "Marie", "Pierre", "Marc", "Jimmy")
  )

  result <- suppressMessages(ljoin_checks(left_table, right_table, by = "person_id"))

  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), nrow(left_table))
  expect_true("person_name" %in% names(result))
  expect_equal(result$person_name[result$person_id == 1][1], "Jimmy")
})

test_that("ljoin_checks emits correct messages", {
  left <- data.frame(id = 1:3, x = c("a", "b", "c"))
  right <- data.frame(id = 2:4, y = c("d", "e", "f"))

  expect_message(ljoin_checks(left, right, by = "id"), "ltable rows")
  expect_message(ljoin_checks(left, right, by = "id"), "rtable rows")
  expect_message(ljoin_checks(left, right, by = "id"), "id are common var names")
})

# WARNING - Generated by {fusen} from dev/flat_utils.Rmd: do not edit by hand

#' Frequencies and proportions in one output
#'
#' Combines base::table() and base::prop.table() outputs in a single one
#'
#' @param ... Arguments passed to \code{base::table()}.
#' @param margin Integer 1L. The same argument as in base::prop.table()
#' @param round Integer 1L. Number of digits after decimal in base::prop.table() output
#' @param noquote Logical 1L. If TRUE, return an object of class noquote that provides better view of the output
#' @return Matrix or noquote matrix. Frequencies with proportions in brackets, within a matrix
#' @examples
#' df <- data.frame(
#'                  "variable_1" = c("v1_1", "v1_1",
#'                                   "v1_2", "v1_2", "v1_2", "v1_2"),
#'                  "variable_2" = c("v2_1", "v2_1", "v2_1", "v2_1",
#'                                   "v2_2", "v2_2")
#'                                   )
#' table_prop(df$variable_1)
#' table_prop(df$variable_1, df$variable_2)
#' table_prop(df$variable_1, df$variable_2, margin = 2, noquote = TRUE)
#' df <- data.frame("person_id" = c(1, 1, 2, 3,
#'                                  2, 4, 5, 5 ,1),
#'                  "person_age" = c(25, 25, 21, 32,
#'                                   21, 48, 50, 50, 52),
#'                  "survey_month" = c("jan", "feb", "mar", "apr",
#'                                     "apr", "may", "jun", "jul", "jan"),
#'                  "survey_answer" = c("no", "yes", "no", "yes",
#'                                      "yes", "yes", "no", "yes", NA))
#'
#' table_prop(df$survey_month)
#' table_prop(df$survey_month, df$survey_answer)
#' table_prop(df$survey_month, df$survey_answer, 
#'            margin = 2, round = 4)
#' @export
#'
table_prop <- function (..., margin = NULL, round = 3, noquote = FALSE) {
  
  table <- table(...)
  ptable <- round(prop.table(table, margin), round)
  
  result <- matrix(paste0(table, " (", ptable, ")"), nrow = nrow(table))
  dimnames(result) <- dimnames(table)
  
  
  if (noquote) noquote(result) else result
}

test_that("table_prop works with single variable", {
  df <- data.frame(
    x = c("a", "a", "b", "b", "b", "c")
  )

  result <- table_prop(df$x)
  expect_true(is.matrix(result))
  expect_equal(nrow(result), 3)
})

test_that("table_prop works with two variables", {
  df <- data.frame(
    variable_1 = c("v1_1", "v1_1", "v1_2", "v1_2", "v1_2", "v1_2"),
    variable_2 = c("v2_1", "v2_1", "v2_1", "v2_1", "v2_2", "v2_2")
  )

  result <- table_prop(df$variable_1, df$variable_2)
  expect_true(is.matrix(result))
  expect_equal(dim(result), c(2, 2))
})

test_that("table_prop margin and rounding", {
  df <- data.frame(
    x = c("a", "a", "b", "b"),
    y = c("c", "d", "c", "d")
  )

  result <- table_prop(df$x, df$y, margin = 2, round = 4)
  expect_true(is.matrix(result))
  # Each proportion within a column should sum to 1
  expect_true(all(grepl("\\(0\\.5\\)", result)))
})

test_that("table_prop noquote option", {
  result <- table_prop(c("a", "a", "b"), noquote = TRUE)
  expect_s3_class(result, "noquote")

  result2 <- table_prop(c("a", "a", "b"), noquote = FALSE)
  expect_false(inherits(result2, "noquote"))
})

# WARNING - Generated by {fusen} from dev/flat_dupl_show.Rmd: do not edit by hand

#' Illustrate sources of all duplicated values of a variable or a combination of variables
#' @param data_frame Data.frame. Input data frame. Must be in the Global Environment and has a data.frame class
#' @param vars Character. Vector of variable or combination of variables
#' from which duplicates are checked
#' @param output_as_df Logical 1L. If TRUE, output is rendered as a data.frame. 
#' @return List or data.frame. For each duplicated row regarding to vars, different values of the same variable
#' are shown, separated by AND
#' @importFrom magrittr %>%
#' @export
#'
#' @examples
#' # A fictional data with duplicated values:
#' df <- data.frame("person_id" = c(1, 1, 2, 3,
#'                                  2, 4, 5, 5 ,1),
#'                  "person_age" = c(25, 25, 21, 32,
#'                                   21, 48, 50, 50, 52),
#'                  "survey_month" = c("jan", "feb", "mar", "apr",
#'                                     "apr", "may", "jun", "jul", "jan"),
#'                  "survey_answer" = c("no", "yes", "no", "yes",
#'                                      "yes", "yes", "no", "yes", NA))
#'
#' # Shuffling observations and columns to make duplicates difficult to see:
#' set.seed(1)
#' df <- df[sample(1:nrow(df)),
#'          sample(1:ncol(df))]
#' df
#'
#' dupl_sources(data_frame = df, vars = "person_id")
#' dupl_sources(data_frame = df, vars = "person_id", output_as_df = TRUE)
dupl_sources <- function (data_frame, vars, output_as_df = FALSE) {
  
  requireNamespace("magrittr")
  # Insert element-wise combination of vars input components within the data:
  # idvarsdupl
  data <- dplyr::mutate(data_frame,
                        idvarsdupl =
                          do.call(what = paste,
                                  purrr::map(vars, function (x)
                                    data_frame[[x]])))
  
  # Filter rows with values of idvarsdupl that appears
  # at least for the second time:
  data %>% dplyr::filter(duplicated(idvarsdupl)) %>%
    
    # extract idvarsdupl variable from the filtered data:
    dplyr::pull(idvarsdupl) %>%
    
    # extract all corresponding observations from data input:
    (function (p) dplyr::filter(data, idvarsdupl %in% p)) %>%
    
    # arrange values to visualize the duplicated cases:
    dplyr::arrange(!!dplyr::sym("idvarsdupl")) %>%
    
    # put vars input to first variable(s) location(s):
    dplyr::relocate(dplyr::all_of(vars)) %>%
    
    (\(d) dplyr::group_by(d, !!dplyr::sym("idvarsdupl")) %>%
       dplyr::summarise_all(\(x) stats::na.omit(unique(x)) %>%
                              paste0(collapse = " [AND] ")) %>%
       dplyr::ungroup() %>%
       apply(1, \(x) c(paste0(x[[1]], collapse = " "),
                       x[stringr::str_detect(x, " \\[AND\\] ")]))) %>%
    purrr::map(as.data.frame) -> result_list
  
  # Make result_list visually comfortable: 
  result_list <- purrr::map(result_list, \(x) {
    
    rownames(x)[1] <- paste(vars, collapse = "-")
    colnames(x) <- "values"
    x
  }) %>% (\(r) {
    stats::setNames(
      r, 
      purrr::map(
        r, 
        \(xr) xr[[1]][1] %>% 
          (\(xr2) glue::glue("Duplicate sources where {paste0(vars, collapse = '-')} equals {xr2}"))))
  })
  
  # Data.frame version of the result: 
  result_df <- result_list %>% 
    purrr::map(t) %>% purrr::map(as.data.frame) %>% 
    purrr::list_rbind() %>% magrittr::set_rownames(NULL)
  
  if (isTRUE(output_as_df)) {
    return(result_df)
  } else return(result_list)
}

test_that("dupl_sources list output", {
  df <- data.frame(
    person_id = c(1, 1, 2, 3, 2, 4, 5, 5, 1),
    person_age = c(25, 25, 21, 32, 21, 48, 50, 50, 52),
    survey_month = c("jan", "feb", "mar", "apr", "apr", "may", "jun", "jul", "jan"),
    survey_answer = c("no", "yes", "no", "yes", "yes", "yes", "no", "yes", NA)
  )
  set.seed(1)
  df <- df[sample(1:nrow(df)), sample(1:ncol(df))]

  result <- dupl_sources(data_frame = df, vars = "person_id")

  expect_true(is.list(result))
  expect_equal(length(result), 3)  # 3 duplicated person_ids: 1, 2, 5
})

test_that("dupl_sources dataframe output", {
  df <- data.frame(
    person_id = c(1, 1, 2, 3, 2, 4, 5, 5, 1),
    person_age = c(25, 25, 21, 32, 21, 48, 50, 50, 52),
    survey_month = c("jan", "feb", "mar", "apr", "apr", "may", "jun", "jul", "jan"),
    survey_answer = c("no", "yes", "no", "yes", "yes", "yes", "no", "yes", NA)
  )
  set.seed(1)
  df <- df[sample(1:nrow(df)), sample(1:ncol(df))]

  result_df <- dupl_sources(data_frame = df, vars = "person_id", output_as_df = TRUE)

  expect_s3_class(result_df, "data.frame")
  expect_equal(nrow(result_df), 3)
})

test_that("dupl_sources compound key", {
  df <- data.frame(
    person_id = c(1, 1, 2, 3, 2, 4, 5, 5, 1),
    person_age = c(25, 25, 21, 32, 21, 48, 50, 50, 52),
    survey_month = c("jan", "feb", "mar", "apr", "apr", "may", "jun", "jul", "jan"),
    survey_answer = c("no", "yes", "no", "yes", "yes", "yes", "no", "yes", NA)
  )
  set.seed(1)
  df <- df[sample(1:nrow(df)), sample(1:ncol(df))]

  result <- dupl_sources(data_frame = df, vars = c("person_id", "survey_month"))

  expect_true(is.list(result))
  expect_true(length(result) >= 1)
})

# WARNING - Generated by {fusen} from dev/flat_chars_structure.Rmd: do not edit by hand

# Characters structure

#' Get :alpha: / :digit: patterns from each symbol of character vector
#' @param input_vector Character. Vector to process
#' @param unique Logical 1L. If TRUE, the result is reduced to unique values.
#' @param named_output Logical 1L. If TRUE, output vector is named after corresponding input values. 
#' @return Character. Vector describing structure of each element of input_vector,
#' see example. 
#' @examples
#' library(magrittr)
#' input <- c("ABC123", "DE4F56", "789GHI", "ABC123")
#'
#' # Default values of unique and named_output: 
#' chars_structure(input_vector = input, unique = TRUE, named_output = TRUE)
#'
#' # unique is set to default value TRUE and named_output is set to FALSE: 
#' chars_structure(input_vector = input, unique = TRUE, named_output = FALSE)
#'
#' # unique is set to FALSE and named_output to FALSE: 
#' chars_structure(input_vector = input, unique = FALSE, named_output = FALSE)
#'
#' # unique is set to FALSE and named_output to defalut value TRUE: 
#' chars_structure(input_vector = input, unique = FALSE, named_output = TRUE)
#' @importFrom magrittr %>%
#' @export
#'
chars_structure <- function (input_vector, 
                             unique = TRUE, named_output = TRUE) {

  requireNamespace("magrittr")

  if (isFALSE(is.character(input_vector))) {
    
    stop("input_vector must be a character vector")
  }
  
  iunique <- unique(input_vector)
  
  result <- iunique %>%
    strsplit("") %>%
    purrr::map(\(x) stringr::str_replace(x, "^[:alpha:]$", "A") %>%
                 stringr::str_replace("^[:digit:]$", "D") %>%
                 stringr::str_replace(" ", "[space]")) %>%
    purrr::map(rle) %>%
    purrr::map(\(x) paste0(x$lengths, x$values) %>%
                 paste(collapse = ", ")) %>%
    unlist()
  
  result_d <- data.frame(
    "input_unique" = iunique, 
    "chars_structure" = result)
  
  if (isTRUE(unique)) {
    if (isTRUE(named_output)) return(stats::setNames(result, iunique)) 
    else return(result)
  } else {
    ljoin_result <- data.frame(
      "input_vector" = input_vector) %>% 
      dplyr::left_join(result_d, by = c("input_vector" = "input_unique"))
    if (isTRUE(named_output)) return(stats::setNames(ljoin_result$chars_structure, 
                                              ljoin_result$input_vector))
    else return(ljoin_result$chars_structure)
  }
}

# WARNING - Generated by {fusen} from dev/flat_chars_structure.Rmd: do not edit by hand

test_that("Valid outputs are consistent", {
  
  library(magrittr)
  input <- c("ABC123", "DE4F56", "789GHI", "ABC123")
  
  # Default values of unique and named_output: 
  unique_named <- chars_structure(input_vector = input, unique = TRUE, named_output = TRUE)
  
  # unique is set to default value TRUE and named_output is set to FALSE: 
  unique_nonamed <- chars_structure(input_vector = input, unique = TRUE, named_output = FALSE)
  
  # unique is set to FALSE and named_output to FALSE: 
  nounique_nonamed <- chars_structure(input_vector = input, unique = FALSE, named_output = FALSE)
  
  # unique is set to FALSE and named_output to defalut value TRUE: 
  nounique_named <- chars_structure(input_vector = input, unique = FALSE, named_output = TRUE)
  
  expect_identical(
    object = unique_named, 
    expected = c(ABC123 = "3A, 3D", 
                 DE4F56 = "2A, 1D, 1A, 2D", 
                 `789GHI` = "3D, 3A"))
  
  expect_identical(
    object = unique_nonamed, 
    expected = c("3A, 3D", "2A, 1D, 1A, 2D", "3D, 3A"))

  expect_identical(
    object = nounique_nonamed, 
    expected = c("3A, 3D", "2A, 1D, 1A, 2D", "3D, 3A", "3A, 3D"))
    
  expect_identical(
    object = nounique_named, 
    expected = c(ABC123 = "3A, 3D", 
                 DE4F56 = "2A, 1D, 1A, 2D", 
                 `789GHI` = "3D, 3A", 
                 ABC123 = "3A, 3D"))
})

test_that("Errors are consistent", {
  
  expect_error(
    object = chars_structure(input_vector = c(1, 2, 3)),
    regexp = "input_vector must be a character vector")
})

# WARNING - Generated by {fusen} from dev/flat_chars_structure_general.Rmd: do not edit by hand

#' Generalized characters structure
#'
#' Get any user-defined patterns from each symbol of character vector
#' @param input_vector Character. Vector to process
#' @param split Character 1L. Symbol separator
#' @param patterns_and_replacements Character. Named character vector of patterns (names) and replacements (values)
#' @param unique Logical 1L. If TRUE, the result is reduced to unique values
#' @param named_output Logical 1L. If TRUE, output vector is named after corresponding input values. 
#' @return A character vector describing the generalized structure of each
#'   element of \code{input_vector}. If \code{named_output} is \code{TRUE},
#'   the vector is named after the corresponding input values. If
#'   \code{unique} is \code{TRUE}, only structures for unique input values
#'   are returned; otherwise one structure per element of \code{input_vector}.
#' @export
#' @examples
#'
#' input <- c("ABC123", "DE4F56", "789GHI", "ABC123")
#'
#' # Default values of unique and named_output:
#' chars_structure_general(input_vector = input, split = "", 
#'                         patterns_and_replacements = c("[:alpha:]" = "[letter]", 
#'                                                       "[:digit:]" = "[number]"), 
#'                         unique = TRUE, named_output = TRUE)
#'
#' # unique is set to default value TRUE and named_output is set to FALSE:
#' chars_structure_general(input_vector = input, split = "", 
#'                         patterns_and_replacements = c("[:alpha:]" = "[letter]", 
#'                                                       "[:digit:]" = "[number]"), 
#'                         unique = TRUE, named_output = FALSE)
#'
#' # unique is set to FALSE and named_output to FALSE:
#' chars_structure_general(input_vector = input, split = "", 
#'                         patterns_and_replacements = c("[:alpha:]" = "[letter]", 
#'                                                       "[:digit:]" = "[number]"), 
#'                         unique = FALSE, named_output = FALSE)
#'
#' # unique is set to FALSE and named_output to defalut value TRUE:
#' chars_structure_general(input_vector = input, split = "", 
#'                         patterns_and_replacements = c("[:alpha:]" = "[letter]", 
#'                                                       "[:digit:]" = "[number]"), 
#'                         unique = FALSE, named_output = TRUE)
chars_structure_general <- function (input_vector, 
                 split = "", 
                 patterns_and_replacements, unique = TRUE, named_output = TRUE) {
  
  if (isFALSE(is.character(input_vector))) {
    
    stop("input_vector must be a character vector")
  }
  
  iunique <- unique(input_vector)
  result <- strsplit(iunique, split)
  result <- lapply(
    result, 
    \(x) {
      stringr::str_replace_all(x, patterns_and_replacements)})
  result <- lapply(result, rle)
  result <- lapply(
    result, 
    \(x) {
      paste(paste0(x$lengths, x$values), collapse = ", ")
    })
  result <- unlist(result)
  result_d <- data.frame(input_unique = iunique, chars_structure = result)
  
  if (unique) {
    if (named_output) {
      stats::setNames(result, iunique)
    } else {result}
  } else {
    ljoin_result <- dplyr::left_join(
      data.frame(input_vector = input_vector), 
      result_d, by = c(input_vector = "input_unique"))
    
    if (named_output) {
      stats::setNames(ljoin_result[["chars_structure"]], 
                      ljoin_result[["input_vector"]])
    } else {ljoin_result[["chars_structure"]]}
  }
}

# WARNING - Generated by {fusen} from dev/flat_chars_structure_general.Rmd: do not edit by hand

test_that("Valid outputs are consistent", {
  
  input <- c("ABC123", "DE4F56", "789GHI", "ABC123")
  
  unique_named <- chars_structure_general(
    input_vector = input, split = "", 
    patterns_and_replacements = c("[:alpha:]" = "[letter]", 
                                  "[:digit:]" = "[number]"), 
    unique = TRUE, named_output = TRUE)
  
  unique_nonamed <- chars_structure_general(
    input_vector = input, split = "", 
    patterns_and_replacements = c("[:alpha:]" = "[letter]", 
                                  "[:digit:]" = "[number]"), 
    unique = TRUE, named_output = FALSE)
  
  nounique_nonamed <- chars_structure_general(
    input_vector = input, split = "", 
    patterns_and_replacements = c("[:alpha:]" = "[letter]", 
                                  "[:digit:]" = "[number]"), 
    unique = FALSE, named_output = FALSE)
  
  nounique_named <- chars_structure_general(
    input_vector = input, split = "", 
    patterns_and_replacements = c("[:alpha:]" = "[letter]", 
                                  "[:digit:]" = "[number]"), 
    unique = FALSE, named_output = TRUE)
  
  expect_identical(
    object = unique_named, 
    expected = c(ABC123 = "3[letter], 3[number]", 
                 DE4F56 = "2[letter], 1[number], 1[letter], 2[number]", 
                 `789GHI` = "3[number], 3[letter]"))
  
  expect_identical(
    object = unique_nonamed, 
    expected = c("3[letter], 3[number]", 
                 "2[letter], 1[number], 1[letter], 2[number]", 
                 "3[number], 3[letter]"))
  
  expect_identical(
    object = nounique_nonamed, 
    expected = c("3[letter], 3[number]", 
                 "2[letter], 1[number], 1[letter], 2[number]", 
                 "3[number], 3[letter]", "3[letter], 3[number]"))
  
  expect_identical(
    object = nounique_named, 
    expected = c(ABC123 = "3[letter], 3[number]", 
                 DE4F56 = "2[letter], 1[number], 1[letter], 2[number]", 
                 `789GHI` = "3[number], 3[letter]", 
                 ABC123 = "3[letter], 3[number]"))
  
})

test_that("Erros are consistent", {
  
  expect_error(
    object = chars_structure_general(input_vector = c(1, 2, 3)), 
    regexp = "input_vector must be a character vector")
})

# WARNING - Generated by {fusen} from dev/flat_convert_r.Rmd: do not edit by hand

#' Create an excel mask compatible with the convert_r() function
#' @param output_path Character 1L. Folder path where the mask will be created
#' @param output_filename Character 1L. File name (with extension) of the mask
#' @return No return value, called for side effects. An Excel file
#'   (\code{.xlsx}) is written to \code{file.path(output_path, output_filename)}
#'   containing an empty template with columns \code{folder_path}, \code{file},
#'   \code{converted_file}, and \code{to_convert}.
#' @export
#'
#' @examples
#' mydir <- file.path(tempdir(), "convert_r_tests_examples")
#' dir.create(mydir)
#'
#' mask_convert_r(output_path = mydir)
#'
#' list.files(mydir)
#'
#' unlink(mydir, recursive = TRUE)
mask_convert_r <- function (output_path,
                            output_filename = "mask_convert_r.xlsx") {

  if (isFALSE(dir.exists(output_path))) {
    
    stop("output_path doesn't exist")
  }
  
  mask <- data.frame(folder_path = NA,
                     file = NA,
                     converted_file = NA,
                     to_convert = NA)

  writexl::write_xlsx(mask,
                      path = file.path(output_path,
                                       output_filename))
}

# WARNING - Generated by {fusen} from dev/flat_convert_r.Rmd: do not edit by hand

test_that("Valid output are consistent", {
  
  mydir <- file.path(tempdir(), "convert_r_tests_examples")
  dir.create(mydir)
  
  mask_convert_r(output_path = mydir)
  
  expect_identical(
    object = list.files(mydir), 
    expected = "mask_convert_r.xlsx")
  
  expect_equal(
    object = names(readxl::read_xlsx(file.path(mydir, "mask_convert_r.xlsx")))[1:4],
    expected = c("folder_path", "file", "converted_file", "to_convert"))
  
  unlink(mydir, recursive = TRUE)
})

test_that("Errors are consistent", {
  
  expect_error(
    mask_convert_r("~/noexist"), 
    "output_path doesn't exist")
})

#' Inspect a data frame
#'
#' Compute a summary of each variable in a data frame: class, distinct values,
#' missing values, void strings, character lengths, and sample modalities.
#'
#' @param data_frame A data frame to inspect.
#' @param nrow Logical. If \code{TRUE}, the number of observations is printed
#'   as a message in addition to the returned table.
#' @return A tibble with one row per variable and columns: \code{variables},
#'   \code{class}, \code{nb_distinct}, \code{prop_distinct}, \code{nb_na},
#'   \code{prop_na}, \code{nb_void}, \code{prop_void}, \code{nchars}, \code{modalities}.
#' @examples
#' inspect(CO2)
#' @importFrom magrittr %>%
#' @export
#'
inspect <- function(data_frame, nrow = FALSE) {

  if (!is.data.frame(data_frame)) {
    stop("data_frame must be an object of data.frame class")
  }

  rows <- nrow(data_frame)
  df <- data_frame %>%
    dplyr::mutate(dplyr::across(
      dplyr::where(lubridate::is.POSIXct),
      \(x) as.character(x) %>% structure(class = "Date-time")
    )) %>%
    purrr::map_df(~ {
      dplyr::tibble(
        class = paste(class(.x), collapse = "/"),
        nb_distinct = dplyr::n_distinct(.x),
        prop_distinct = nb_distinct / rows,
        nb_na = sum(is.na(.x)),
        prop_na = nb_na / rows,
        nb_void = sum(.x == "", na.rm = TRUE),
        prop_void = nb_void / rows,
        nchars = paste(unique(sort(nchar(as.character(.x))))[
          1:min(dplyr::n_distinct(nchar(as.character(.x))), 10)],
          collapse = " / "),
        modalities = paste(sort(unique(.x))[
          1:min(dplyr::n_distinct(.x), 10)],
          collapse = " / ")
      )
    }, .id = "variables")

  if (nrow) {
    message("Number of observations: ", rows)
  }

  df
}

test_that("inspect returns correct structure", {
  result <- inspect(CO2)

  expect_s3_class(result, "tbl_df")
  expect_true(all(c("variables", "class", "nb_distinct", "prop_distinct",
                     "nb_na", "prop_na", "nb_void", "prop_void",
                     "nchars", "modalities") %in% names(result)))
  expect_equal(nrow(result), ncol(CO2))
})

test_that("inspect values are correct for CO2", {
  result <- inspect(CO2)

  expect_equal(result$variables, names(CO2))
  expect_equal(result$nb_na, rep(0L, ncol(CO2)))
  expect_equal(result$nb_void, rep(0L, ncol(CO2)))
  expect_equal(result$nb_distinct[result$variables == "Type"], 2L)
  expect_equal(result$nb_distinct[result$variables == "Treatment"], 2L)
})

test_that("inspect detects NAs and voids", {
  df <- data.frame(
    x = c(1, NA, 3, NA),
    y = c("a", "", "c", ""),
    stringsAsFactors = FALSE
  )
  result <- inspect(df)

  expect_equal(result$nb_na[result$variables == "x"], 2L)
  expect_equal(result$nb_void[result$variables == "y"], 2L)
  expect_equal(result$prop_na[result$variables == "x"], 0.5)
  expect_equal(result$prop_void[result$variables == "y"], 0.5)
})

test_that("inspect nrow parameter emits message", {
  expect_message(inspect(cars, nrow = TRUE), "Number of observations: 50")
})

test_that("inspect handles POSIXct columns", {
  df <- data.frame(
    ts = as.POSIXct(c("2024-01-01", "2024-06-15")),
    val = c(1, 2)
  )
  result <- inspect(df)

  expect_equal(nrow(result), 2)
  expect_true(any(grepl("Date-time", result$class)))
})

test_that("inspect errors on non-dataframe", {
  expect_error(inspect(as.matrix(cars)),
               "data_frame must be an object of data.frame class")
})

#' Inspect a collection of datasets
#'
#' Read all datasets from a folder matching the specified extensions, inspect each one,
#' and write a comprehensive Excel report covering variable presence, types, and per-dataset summaries.
#'
#' @param input_path Character. Folder path containing datasets to explore.
#' @param output_path Character. Folder path where the Excel output will be written.
#' @param output_label Character. A concise label for the output file name.
#' @param considered_extensions Character vector. File extensions to include
#'   (without the dot, e.g. \code{"rds"} not \code{".rds"}).
#' @param encoding Character 1L or \code{NULL}. Encoding passed to
#'   \code{rio::import()} for text-based formats (CSV, TSV). Must be one of
#'   \code{"unknown"}, \code{"UTF-8"}, or \code{"Latin-1"}. Defaults to
#'   \code{NULL} (let \code{rio} / \code{data.table} pick the default).
#'   Ignored for binary formats (e.g. RDS, xlsx).
#' @return Invisibly returns the path to the written Excel file.
#'   The file contains sheets: dims, inspect_tot, one per dataset inspection,
#'   vars_detect, vars_detect_everywhere, vars_detect_not_everywhere,
#'   vars_compclasses, vars_compclasses_allsame, vars_compclasses_not_allsame.
#' @examples
#' mydir <- file.path(tempdir(), "inspect_vars_example")
#' dir.create(mydir)
#'
#' saveRDS(cars, file.path(mydir, "cars1.rds"))
#' saveRDS(mtcars, file.path(mydir, "cars2.rds"))
#'
#' inspect_vars(input_path = mydir, output_path = mydir,
#'              output_label = "cardata",
#'              considered_extensions = "rds")
#'
#' # Read back the 10 sheets:
#' purrr::map(1:10, function(x)
#'            rio::import(file.path(mydir, "inspect_vars_cardata.xlsx"),
#'                        sheet = x))
#'
#' unlink(mydir, recursive = TRUE)
#' @importFrom magrittr %>%
#' @importFrom stats nobs
#' @export
#'
inspect_vars <- function(input_path, output_path,
                         output_label, considered_extensions,
                         encoding = NULL) {

  if (!is.null(encoding) &&
      !encoding %in% c("unknown", "UTF-8", "Latin-1")) {
    stop("encoding must be NULL or one of 'unknown', 'UTF-8', 'Latin-1'")
  }

  # Build file list
  ext <- paste0("\\.", considered_extensions, "$") %>%
    paste0(collapse = "|")
  lfiles <- list.files(input_path) %>%
    purrr::keep(stringr::str_detect(., ext))

  if (length(lfiles) == 0) {
    stop("No files found in input_path matching the specified extensions")
  }

  # Import all datasets into a LOCAL named list (no global env pollution).
  # Encoding is only forwarded when the user set it explicitly AND the file
  # is a text-based format; otherwise rio uses its own defaults.
  text_exts <- c("csv", "tsv", "txt", "csv2", "psv")
  import_one <- function(f) {
    fpath <- file.path(input_path, f)
    f_ext <- tolower(tools::file_ext(f))
    if (!is.null(encoding) && f_ext %in% text_exts) {
      rio::import(fpath, encoding = encoding, trust = TRUE)
    } else {
      rio::import(fpath, trust = TRUE)
    }
  }

  datasets <- purrr::map(lfiles, import_one) %>%
    stats::setNames(lfiles)

  # Clean names (without extension) for labeling
  clean_names <- tools::file_path_sans_ext(lfiles)
  named_datasets <- stats::setNames(datasets, clean_names)

  # Compute inspection for each dataset
  inspections <- purrr::map(datasets, inspect)

  # Dimensions summary
  dims <- purrr::map_dfr(seq_along(datasets), function(idx) {
    df <- datasets[[idx]]
    tibble::tibble(
      datasets = clean_names[idx],
      nobs = nrow(df),
      nvar = ncol(df)
    )
  })

  # Combined inspection table
  inspect_tot <- purrr::map_dfr(seq_along(inspections), function(idx) {
    inspections[[idx]] %>%
      dplyr::mutate(datasets = clean_names[idx])
  }) %>%
    dplyr::relocate(datasets) %>%
    dplyr::left_join(dims, by = "datasets") %>%
    dplyr::relocate(nvar, nobs, .after = datasets)

  # Individual inspection sheets (with prepended dims rows for backward compat)
  inspect_sheets <- purrr::map(seq_along(inspections), function(idx) {
    df <- datasets[[idx]]
    i <- inspections[[idx]]
    rbind(
      c("Obs = ", nrow(df), rep("", ncol(i) - 1)),
      c("Nvars = ", nrow(i), rep("", ncol(i) - 1)),
      cbind(1:nrow(i), i)
    )
  }) %>%
    stats::setNames(clean_names)

  # Variable detection and type comparison
  vars_detect_data <- vars_detect(named_datasets)
  vars_detect_everywhere_data <- vars_detect_everywhere(vars_detect_data)
  vars_detect_not_everywhere_data <- vars_detect_not_everywhere(vars_detect_data)

  vars_compclasses_data <- vars_compclasses(named_datasets)
  # Reorder to match vars_detect row order
  vars_compclasses_data <- vars_compclasses_data[
    order(match(vars_compclasses_data$vars_union,
                vars_detect_data$vars_union)), ]

  vars_compclasses_allsame_data <- vars_compclasses_allsame(vars_compclasses_data)
  vars_compclasses_not_allsame_data <- vars_compclasses_not_allsame(vars_compclasses_data)

  # Assemble all sheets
  sheets <- c(
    list(dims = dims, inspect_tot = inspect_tot),
    inspect_sheets,
    list(
      vars_detect = vars_detect_data,
      vars_detect_everywhere = vars_detect_everywhere_data,
      vars_detect_not_everywhere = vars_detect_not_everywhere_data,
      vars_compclasses = vars_compclasses_data,
      vars_compclasses_allsame = vars_compclasses_allsame_data,
      vars_compclasses_not_allsame = vars_compclasses_not_allsame_data
    )
  )

  out_path <- file.path(output_path, paste0("inspect_vars_", output_label, ".xlsx"))
  writexl::write_xlsx(sheets, out_path)

  invisible(out_path)
}

#' Replace character vector values using a correspondence approach
#'
#' Names of \code{replacements} are matched literally (not as regular
#' expressions). Elements of \code{input_vector} that match none of the
#' patterns are returned unchanged.
#'
#' @param input_vector Character. Character vector on which replacements take place.
#' @param replacements Character. Named character vector defining replacement
#'   correspondences (names = patterns, values = replacements). Names are
#'   treated as literal strings (regex metacharacters such as \code{.},
#'   \code{(}, \code{+} are not interpreted).
#' @param replace_all Logical. If \code{TRUE}, \code{stringr::str_replace_all()}
#'   is used instead of \code{stringr::str_replace()}.
#' @return Character vector with replacements applied.
#' @section Overlapping patterns:
#' When several patterns can match the same element, the first match found
#' in \code{input_vector} (scanned left to right) is used. When patterns
#' start at the same position, the one listed first in \code{replacements}
#' wins, not the longest. Order \code{replacements} accordingly if some
#' patterns are prefixes of others.
#' @examples
#' input <- c("one-one", "two-two-one", "three-three-two")
#'
#' replace_multiple(input,
#'                  replacements =
#'                    c("one" = "1", "two" = "2",
#'                      "three" = "3"))
#'
#' replace_multiple(input,
#'                  replacements =
#'                    c("one" = "1", "two" = "2",
#'                      "three" = "3"),
#'                  replace_all = TRUE)
#'
#' # Unmatched elements are returned as-is:
#' replace_multiple(c("one", "unmatched"), c("one" = "1"))
#'
#' # Regex metacharacters are matched literally:
#' replace_multiple(c("a.b", "aXb"), c("." = "DOT"))
#' @importFrom magrittr %>%
#' @export
#'
replace_multiple <- function(input_vector, replacements,
                             replace_all = FALSE) {

  if (!is.character(input_vector)) {
    stop("input_vector must be a character vector")
  }

  if (!is.character(replacements)) {
    stop("replacements must be a character vector")
  }

  replace_function <- function(...) {
    if (replace_all) stringr::str_replace_all(...)
    else stringr::str_replace(...)
  }

  input <- names(replacements)

  if (length(unique(input)) != length(input)) {
    stop("Original values must be unique in the `replacements` argument.")
  }

  output <- unname(replacements)

  replace_table <- data.frame(input, output)

  # Escape regex metacharacters so that names of `replacements` are matched
  # as literal strings both during extraction and during replacement.
  escaped_input <- stringr::str_escape(input)

  dplyr::tibble(input_vector = input_vector,
                origin =
                  stringr::str_extract(input_vector,
                                       escaped_input %>% paste(collapse = "|"))) %>%
    dplyr::left_join(replace_table, by = c("origin" = "input")) %>%
    dplyr::mutate(destination = dplyr::if_else(
      is.na(origin),
      input_vector,
      replace_function(input_vector, stringr::fixed(origin), output))) %>%
    dplyr::pull(destination)
}

test_that("replace_multiple first occurrence", {
  input <- c("one-one", "two-two-one", "three-three-two")

  expect_identical(
    replace_multiple(input, replacements = c("one" = "1", "two" = "2", "three" = "3")),
    c("1-one", "2-two-one", "3-three-two")
  )
})

test_that("replace_multiple all occurrences", {
  input <- c("one-one", "two-two-one", "three-three-two")

  expect_identical(
    replace_multiple(input, replacements = c("one" = "1", "two" = "2", "three" = "3"),
                     replace_all = TRUE),
    c("1-1", "2-2-one", "3-3-two")
  )
})

test_that("replace_multiple errors on invalid input", {
  expect_error(replace_multiple(1:2, c("A" = "B")),
               "input_vector must be a character vector")

  expect_error(replace_multiple(c("A", "B"), 1:2),
               "replacements must be a character vector")
})

test_that("replace_multiple errors on duplicate patterns", {
  expect_error(
    replace_multiple(c("a", "b"), c("a" = "1", "a" = "2")),
    "Original values must be unique"
  )
})

# WARNING - Generated by {fusen} from dev/flat_rename_r.Rmd: do not edit by hand

#' Industrialized file renaming
#' 
#' @param mask_filepath Character 1L. Entire file path of the excel mask
#'
#' @return No return value, called for side effects. Files are renamed
#'   on disk according to the instructions in the Excel mask.
#' 
#' @export
#' @examples
#' library(magrittr)
#' data(cars)
#' data(mtcars)
#'
#' mydir <- tempfile()
#' dir.create(mydir)
#'
#' # Two example files to rename: 
#' saveRDS(cars, file.path(mydir, "cars.rds"))
#' saveRDS(mtcars, file.path(mydir, "mtcars.rds"))
#' list.files(mydir)
#'
#' # Create the mask: 
#' mask_rename_r(input_path = mydir)
#'
#' # Fill the mask (in practice you can do it manually): 
#' mask <- rio::import(file.path(mydir, "mask_rename_r.xlsx"))
#' mask[["renamed_file"]] <- c("cars_renamed.rds", "mtcars_renamed.rds")
#' mask[["to_rename"]] <- rep(1, 2)
#' writexl::write_xlsx(mask, file.path(mydir, "mask_rename_r.xlsx"))
#'
#' # Apply the rename function: 
#' rename_r(mask_filepath = file.path(mydir, "mask_rename_r.xlsx"))
#'
#' # See the renamed files: 
#' list.files(mydir)
#'
#' # Clean tempdir: 
#' unlink(mydir, recursive = TRUE)
rename_r <- function(mask_filepath){
    
  if (! file.exists(mask_filepath)) {
    stop("mask_filepath doesn't exist. It must be a valid and full path including the xlsx file.")
  }
  
  prm <- rio::import(mask_filepath)
  
  if (! all(c("file", "renamed_file", "to_rename") %in% names(prm))) {
    stop("The mask must have the columns: file, renamed_file, to_rename")
  }
  
  prm <- prm[prm$to_rename == 1, ]
  prm[["row_number"]] <- 1:nrow(prm)
  
  mask_dir <- dirname(mask_filepath)
  errors <- character(0)

  results <- purrr::map(seq_len(nrow(prm)), \(i) {
    row <- prm[i, ]
    tryCatch({
      ok <- file.rename(
        file.path(mask_dir, row[["file"]]),
        file.path(mask_dir, row[["renamed_file"]])
      )
      if (!ok) stop("file.rename returned FALSE")
      message("Renamed: ", row[["file"]], " -> ", row[["renamed_file"]])
      TRUE
    }, error = function(e) {
      errors <<- c(errors, row[["file"]])
      warning("  Failed: ", row[["file"]], " - ", conditionMessage(e),
              call. = FALSE)
      FALSE
    })
  })

  if (length(errors) > 0) {
    warning(length(errors), " file(s) failed: ",
            paste(errors, collapse = ", "), call. = FALSE)
  }

  invisible(results)
}

#' Batch dataset format conversion using an Excel mask
#'
#' Read an Excel mask describing files to convert, then import, clean, and
#' re-export each file in the desired format. Cleaning includes replacing void
#' character values (\code{""}) with \code{NA} and trimming whitespace.
#'
#' @param mask_filepath Character. Full file path to the Excel mask.
#'   The mask must contain columns: \code{folder_path}, \code{file},
#'   \code{converted_file}, \code{to_convert} (1 to convert, 0 to skip).
#' @param output_path Character. Folder path where converted files will be placed.
#' @return Invisibly returns a character vector of output file paths.
#' @importFrom magrittr %>%
#' @export
#'
#' @examples
#' mydir <- system.file("permadir_examples_and_tests/convert_r", package = "scrutr")
#'
#' mask <- data.frame(
#'   folder_path = rep(mydir, 2),
#'   file = c("original_cars.rds", "original_mtcars.csv"),
#'   converted_file = c("converted_cars.csv", "converted_mtcars.csv"),
#'   to_convert = rep(1, 2)
#' )
#'
#' \donttest{
#' mask_path <- file.path(tempdir(), "mask_convert_r.xlsx")
#' writexl::write_xlsx(mask, mask_path)
#'
#' convert_r(
#'   mask_filepath = mask_path,
#'   output_path = tempdir()
#' )
#'
#' # Clean up:
#' file.remove(file.path(tempdir(),
#'   c("converted_cars.csv", "converted_mtcars.csv", "mask_convert_r.xlsx")))
#' }
convert_r <- function(mask_filepath, output_path) {

  if (!file.exists(mask_filepath)) {
    stop("mask_filepath doesn't exist. It must be a valid and full path including the xlsx file.")
  }

  if (!dir.exists(output_path)) {
    stop("output_path doesn't exist. Check path validity.")
  }

  prm <- rio::import(mask_filepath)
  prm <- dplyr::filter(prm, to_convert == 1)

  if (nrow(prm) == 0) {
    message("No files marked for conversion (to_convert == 1).")
    return(invisible(character(0)))
  }

  out_paths <- character(nrow(prm))
  errors <- character(0)

  for (i in seq_len(nrow(prm))) {
    row <- prm[i, ]
    message("Converting: ", row$file)

    tryCatch({
      data <- rio::import(file.path(row$folder_path, row$file), trust = TRUE)
      data <- dplyr::mutate(data, dplyr::across(
        dplyr::everything(),
        \(y) { y[nchar(as.character(y)) == 0] <- NA; y }
      ))
      data <- dplyr::mutate(data, dplyr::across(
        dplyr::where(is.character),
        stringr::str_trim
      ))

      out_file <- file.path(output_path, row$converted_file)
      rio::export(data, out_file)
      out_paths[i] <- out_file
      message("  Done: ", row$converted_file)
    }, error = function(e) {
      errors <<- c(errors, row$file)
      warning("  Failed: ", row$file, " - ", conditionMessage(e),
              call. = FALSE)
    })
  }

  if (length(errors) > 0) {
    warning(length(errors), " file(s) failed: ",
            paste(errors, collapse = ", "), call. = FALSE)
  }

  invisible(out_paths)
}

test_that("convert_r converts files correctly", {
  mydir <- system.file("permadir_examples_and_tests/convert_r", package = "scrutr")
  outdir <- file.path(tempdir(), "test_convert_r")
  dir.create(outdir, showWarnings = FALSE)
  on.exit(unlink(outdir, recursive = TRUE))

  mask <- data.frame(
    folder_path = rep(mydir, 2),
    file = c("original_cars.rds", "original_mtcars.csv"),
    converted_file = c("converted_cars.csv", "converted_mtcars.rds"),
    to_convert = rep(1, 2)
  )
  mask_path <- file.path(outdir, "mask.xlsx")
  writexl::write_xlsx(mask, mask_path)

  result <- convert_r(mask_filepath = mask_path, output_path = outdir)

  expect_true(file.exists(file.path(outdir, "converted_cars.csv")))
  expect_true(file.exists(file.path(outdir, "converted_mtcars.rds")))

  # Verify content is preserved
  converted_cars <- rio::import(file.path(outdir, "converted_cars.csv"))
  expect_equal(nrow(converted_cars), 50)
  expect_equal(ncol(converted_cars), 2)
})

test_that("convert_r handles to_convert filter", {
  mydir <- system.file("permadir_examples_and_tests/convert_r", package = "scrutr")
  outdir <- file.path(tempdir(), "test_convert_r_filter")
  dir.create(outdir, showWarnings = FALSE)
  on.exit(unlink(outdir, recursive = TRUE))

  mask <- data.frame(
    folder_path = rep(mydir, 2),
    file = c("original_cars.rds", "original_mtcars.csv"),
    converted_file = c("converted_cars.csv", "converted_mtcars.csv"),
    to_convert = c(1, 0)  # Only convert first
  )
  mask_path <- file.path(outdir, "mask.xlsx")
  writexl::write_xlsx(mask, mask_path)

  convert_r(mask_filepath = mask_path, output_path = outdir)

  expect_true(file.exists(file.path(outdir, "converted_cars.csv")))
  expect_false(file.exists(file.path(outdir, "converted_mtcars.csv")))
})

test_that("convert_r handles empty mask", {
  mydir <- system.file("permadir_examples_and_tests/convert_r", package = "scrutr")
  outdir <- file.path(tempdir(), "test_convert_r_empty")
  dir.create(outdir, showWarnings = FALSE)
  on.exit(unlink(outdir, recursive = TRUE))

  mask <- data.frame(
    folder_path = mydir,
    file = "original_cars.rds",
    converted_file = "converted.csv",
    to_convert = 0
  )
  mask_path <- file.path(outdir, "mask.xlsx")
  writexl::write_xlsx(mask, mask_path)

  expect_message(convert_r(mask_filepath = mask_path, output_path = outdir),
                 "No files marked for conversion")
})

test_that("convert_r errors on invalid paths", {
  expect_error(
    convert_r("~/noexist/mask.xlsx", output_path = tempdir()),
    "mask_filepath doesn't exist"
  )

  mydir <- system.file("permadir_examples_and_tests/convert_r", package = "scrutr")
  mask <- data.frame(
    folder_path = mydir,
    file = "original_cars.rds",
    converted_file = "converted.csv",
    to_convert = 1
  )
  mask_path <- file.path(tempdir(), "test_mask_err.xlsx")
  writexl::write_xlsx(mask, mask_path)
  on.exit(file.remove(mask_path), add = TRUE)

  expect_error(
    convert_r(mask_path, paste0(tempdir(), "/noexist")),
    "output_path doesn't exist"
  )
})

# WARNING - Generated by {fusen} from dev/flat_paths_and_files.Rmd: do not edit by hand

#' Replicate the folder structure of a given directory
#'
#' @param dir Character 1L. Path of directory which structure will be replicated
#' @param to Character 1L. Path of an output directory in which replicated structured will be placed
#' @return No return value, called for side effects. The directory
#'   structure of \code{dir} is recreated inside the \code{to} directory.
#' @importFrom magrittr %>%
#' @export
#'
#' @examples
#' library(magrittr)
#'
#' temp_dir_to_replicate <- tempfile()
#' dir.create(temp_dir_to_replicate)
#'
#' dir.create(file.path(temp_dir_to_replicate, "dir1"))
#' dir.create(file.path(temp_dir_to_replicate, "dir2"))
#'
#' temp_dir_out <- tempfile()
#' dir.create(temp_dir_out)
#'
#' folder_structure_replicate(
#'   dir = temp_dir_to_replicate, 
#'   to = temp_dir_out)
#'
#' unlink(temp_dir_to_replicate)
#' unlink(temp_dir_out)
folder_structure_replicate <- function (dir, to) {

  requireNamespace("magrittr")

  if (isTRUE(identical(dir, to))) {
    
    stop(glue::glue("The arguments dir and to must not be identical"))
  }
  
  if (isTRUE(
    length(list.dirs(to, recursive = FALSE)) > 0)) {
    
    stop(glue::glue("For security reasons, {to} must be an empty directory"))
  }
  
  # Warnings are suppressed because of the two security checks above: 
  suppressWarnings({
    
    list.dirs(dir, full.names = FALSE) %>%
      (\(l) for (i in l)
        dir.create(file.path(to, i), recursive = TRUE))
  })
  
}

test_that("folder_structure_replicate copies structure", {
  temp_src <- tempfile()
  dir.create(temp_src)
  dir.create(file.path(temp_src, "dir1"))
  dir.create(file.path(temp_src, "dir2"))
  dir.create(file.path(temp_src, "dir2", "subdir"))

  temp_out <- tempfile()
  dir.create(temp_out)

  folder_structure_replicate(dir = temp_src, to = temp_out)

  src_dirs <- list.dirs(temp_src, full.names = FALSE, recursive = TRUE)
  out_dirs <- list.dirs(temp_out, full.names = FALSE, recursive = TRUE)

  expect_equal(sort(src_dirs), sort(out_dirs))

  unlink(c(temp_src, temp_out), recursive = TRUE)
})

test_that("folder_structure_replicate errors on identical paths", {
  temp <- tempfile()
  dir.create(temp)
  on.exit(unlink(temp, recursive = TRUE))

  expect_error(
    folder_structure_replicate(dir = temp, to = temp),
    "must not be identical"
  )
})

test_that("folder_structure_replicate errors on non-empty target", {
  temp_src <- tempfile()
  dir.create(temp_src)
  dir.create(file.path(temp_src, "dir1"))

  temp_out <- tempfile()
  dir.create(temp_out)
  dir.create(file.path(temp_out, "existing"))
  on.exit(unlink(c(temp_src, temp_out), recursive = TRUE))

  expect_error(
    folder_structure_replicate(dir = temp_src, to = temp_out),
    "must be an empty directory"
  )
})

#' Variable detection - presence across all datasets
#'
#' @param vars_detect_table Output of the \code{vars_detect()} function.
#' @return A subset of \code{vars_detect_table} containing only variables
#'   present in all datasets.
#' @examples
#' data_list <- list(cars = cars, mtcars = mtcars)
#' vdetect_table <- vars_detect(data_list)
#' vars_detect_everywhere(vdetect_table)
#' @importFrom magrittr %>%
#' @export
#'
vars_detect_everywhere <- function(vars_detect_table) {

  everywhere <- apply(vars_detect_table, 1,
                      \(x) identical(unique(x[-1]), "ok")) %>% unlist

  vars_detect_table[everywhere, ]
}

test_that("vars_detect_everywhere returns common variables only", {
  data_list <- list(cars = cars, mtcars = mtcars)
  vdetect_table <- vars_detect(data_list)

  result <- vars_detect_everywhere(vdetect_table)

  # cars and mtcars have no common variables
  expect_equal(nrow(result), 0)
  expect_true("vars_union" %in% names(result))
})

test_that("vars_detect_everywhere with overlapping variables", {
  df1 <- data.frame(a = 1, b = 2, c = 3)
  df2 <- data.frame(b = 4, c = 5, d = 6)

  vdetect_table <- vars_detect(list(df1 = df1, df2 = df2))
  result <- vars_detect_everywhere(vdetect_table)

  # b and c are in both
  expect_equal(nrow(result), 2)
  expect_true(all(c("b", "c") %in% result$vars_union))
})

#' Variable detection patterns
#'
#' Detect the presence or absence of each variable across a collection of datasets.
#'
#' @param data_frames A named list of data frames to compare.
#' @return A data frame with a \code{vars_union} column listing all variables,
#'   and one column per dataset indicating \code{"ok"} (present) or \code{"-"} (absent).
#'   Rows are sorted to highlight presence/absence patterns.
#' @examples
#' data_list <- list(cars = cars, mtcars = mtcars)
#' vars_detect(data_list)
#' @importFrom magrittr %>%
#' @export
#'
vars_detect <- function(data_frames) {

  if (!is.list(data_frames) || is.data.frame(data_frames)) {
    stop("data_frames must be a named list of data frames")
  }
  if (is.null(names(data_frames)) || any(names(data_frames) == "")) {
    stop("data_frames must be a named list (all elements must have names)")
  }

  # Get variable names per dataset

  var_lists <- purrr::map(data_frames, names)

  # Union of all variable names

  all_vars <- unique(unlist(var_lists))

  # Build presence/absence matrix
  presence <- purrr::map_dfc(var_lists, function(vars) {
    ifelse(all_vars %in% vars, "ok", "-")
  })

  out_data <- dplyr::bind_cols(
    tibble::tibble(vars_union = all_vars),
    presence
  )

  # Arrange lines to better visualize presence/absence patterns
  suppressWarnings({
    rkfirst_ok <- out_data[, -1, drop = FALSE] %>%
      apply(1, \(x) min(which(x == "ok")))

    nb_ok_conseq <- out_data %>%
      apply(1, \(x) rle(as.numeric(x == "ok"))$lengths[2])

    rkfirst_out <- out_data[, -1, drop = FALSE] %>%
      apply(1, \(x) min(which(x == "-")))

    nb_out_conseq <- out_data %>%
      apply(1, \(x) rle(as.numeric(x == "-"))$lengths[2])
  })

  out_data <- dplyr::mutate(out_data,
                            rkfirst_ok, nb_ok_conseq,
                            rkfirst_out, nb_out_conseq)

  dplyr::arrange(out_data,
                 rkfirst_ok, dplyr::desc(nb_ok_conseq),
                 dplyr::desc(rkfirst_out), dplyr::desc(nb_out_conseq)) %>%
    dplyr::select(-rkfirst_ok, -nb_ok_conseq,
                  -rkfirst_out, -nb_out_conseq)
}

test_that("vars_detect works with named list", {
  data_list <- list(cars = cars, mtcars = mtcars)

  result <- vars_detect(data_list)

  expect_s3_class(result, "data.frame")
  expect_true("vars_union" %in% names(result))
  expect_true("cars" %in% names(result))
  expect_true("mtcars" %in% names(result))

  # cars has speed and dist; mtcars does not

  expect_equal(result$cars[result$vars_union == "speed"], "ok")
  expect_equal(result$cars[result$vars_union == "dist"], "ok")
  expect_equal(result$mtcars[result$vars_union == "speed"], "-")

  # mtcars has mpg; cars does not
  expect_equal(result$mtcars[result$vars_union == "mpg"], "ok")
  expect_equal(result$cars[result$vars_union == "mpg"], "-")

  # All 13 variables (2 from cars + 11 from mtcars, no overlap)
  expect_equal(nrow(result), 13)
})

test_that("vars_detect with overlapping variables", {
  df1 <- data.frame(a = 1, b = 2, c = 3)
  df2 <- data.frame(b = 4, c = 5, d = 6)

  result <- vars_detect(list(df1 = df1, df2 = df2))

  expect_equal(nrow(result), 4)  # a, b, c, d
  expect_equal(result$df1[result$vars_union == "a"], "ok")
  expect_equal(result$df2[result$vars_union == "a"], "-")
  expect_equal(result$df1[result$vars_union == "b"], "ok")
  expect_equal(result$df2[result$vars_union == "b"], "ok")
  expect_equal(result$df1[result$vars_union == "d"], "-")
  expect_equal(result$df2[result$vars_union == "d"], "ok")
})

test_that("vars_detect errors on invalid input", {
  expect_error(vars_detect(cars), "named list of data frames")
  expect_error(vars_detect(list(cars, mtcars)), "named list")
  expect_error(vars_detect("cars"), "named list of data frames")
})

#' Variable detection - inconsistent patterns
#'
#' @param vars_detect_table Output of the \code{vars_detect()} function.
#' @return A subset of \code{vars_detect_table} containing only variables
#'   that are not present in every dataset.
#' @examples
#' data_list <- list(cars = cars, mtcars = mtcars)
#' vdetect_table <- vars_detect(data_list)
#' vars_detect_not_everywhere(vdetect_table)
#' @importFrom magrittr %>%
#' @export
#'
vars_detect_not_everywhere <- function(vars_detect_table) {

  not_everywhere <- apply(vars_detect_table, 1,
                          \(x) !identical(unique(x[-1]), "ok")) %>% unlist

  vars_detect_table[not_everywhere, ]
}

#' Collection-level variables types comparison
#'
#' Compare the class of each variable across a collection of datasets.
#'
#' @param data_frames A named list of data frames to compare.
#' @return A data frame with a \code{vars_union} column and one column per dataset
#'   showing the class of each variable (\code{"-"} if absent from that dataset).
#' @examples
#' data_list <- list(cars = cars, mtcars = mtcars)
#' vars_compclasses(data_list)
#' @importFrom magrittr %>%
#' @export
#'
vars_compclasses <- function(data_frames) {

  if (!is.list(data_frames) || is.data.frame(data_frames)) {
    stop("data_frames must be a named list of data frames")
  }
  if (is.null(names(data_frames)) || any(names(data_frames) == "")) {
    stop("data_frames must be a named list (all elements must have names)")
  }

  all_vars <- unique(unlist(purrr::map(data_frames, names)))

  classes <- purrr::map_dfc(data_frames, function(df) {
    purrr::map_chr(all_vars, function(v) {
      if (v %in% names(df)) {
        paste(class(df[[v]]), collapse = "/")
      } else {
        "-"
      }
    })
  })

  dplyr::bind_cols(
    tibble::tibble(vars_union = all_vars),
    classes
  )
}

test_that("vars_compclasses works with named list", {
  data_list <- list(cars = cars, mtcars = mtcars)

  result <- vars_compclasses(data_list)

  expect_s3_class(result, "data.frame")
  expect_true("vars_union" %in% names(result))
  expect_equal(nrow(result), 13)

  # speed is numeric in cars, absent from mtcars
  expect_equal(result$cars[result$vars_union == "speed"], "numeric")
  expect_equal(result$mtcars[result$vars_union == "speed"], "-")

  # mpg is numeric in mtcars, absent from cars
  expect_equal(result$mtcars[result$vars_union == "mpg"], "numeric")
  expect_equal(result$cars[result$vars_union == "mpg"], "-")
})

test_that("vars_compclasses detects different types", {
  df1 <- data.frame(x = 1:3, y = c("a", "b", "c"), stringsAsFactors = FALSE)
  df2 <- data.frame(x = c(1.1, 2.2, 3.3), y = factor(c("a", "b", "c")))

  result <- vars_compclasses(list(df1 = df1, df2 = df2))

  expect_equal(result$df1[result$vars_union == "x"], "integer")
  expect_equal(result$df2[result$vars_union == "x"], "numeric")
  expect_equal(result$df1[result$vars_union == "y"], "character")
  expect_equal(result$df2[result$vars_union == "y"], "factor")
})

test_that("vars_compclasses errors on invalid input", {
  expect_error(vars_compclasses(cars), "named list of data frames")
  expect_error(vars_compclasses(list(cars, mtcars)), "named list")
})

test_that("vars_compclasses_allsame filters consistent types", {
  data_list <- list(cars = cars, mtcars = mtcars)
  vcompclasses <- vars_compclasses(data_list)

  result <- vars_compclasses_allsame(vcompclasses)

  # All variables in cars/mtcars are numeric; absent vars are "-"
  # Variables with only one non-"-" class are "allsame"
  expect_equal(nrow(result), 13)
})

test_that("vars_compclasses_allsame with mixed types", {
  df1 <- data.frame(x = 1:3, y = c("a", "b", "c"), stringsAsFactors = FALSE)
  df2 <- data.frame(x = c(1.1, 2.2, 3.3), y = factor(c("a", "b", "c")))

  vcompclasses <- vars_compclasses(list(df1 = df1, df2 = df2))
  result <- vars_compclasses_allsame(vcompclasses)

  # Neither x (integer vs numeric) nor y (character vs factor) are allsame
  expect_equal(nrow(result), 0)
})

test_that("vars_compclasses_allsame with consistent types", {
  df1 <- data.frame(x = 1.0, y = 2.0)
  df2 <- data.frame(x = 3.0, y = 4.0)

  vcompclasses <- vars_compclasses(list(df1 = df1, df2 = df2))
  result <- vars_compclasses_allsame(vcompclasses)

  expect_equal(nrow(result), 2)
})

#' Convert all datasets within a folder to a given file format
#'
#' Scan a folder for files matching specified extensions, then convert each one
#' to the target format. Void character values are replaced with \code{NA} and
#' character whitespace is trimmed.
#'
#' @param input_folderpath Character. Folder path containing datasets to convert.
#' @param considered_extensions Character vector. File extensions to include
#'   (must be supported by \pkg{rio}).
#' @param to Character. The target output format (must be supported by \pkg{rio}).
#' @param output_folderpath Character. Folder path for converted files
#'   (defaults to \code{input_folderpath}).
#' @return Invisibly returns a character vector of output file paths.
#' @export
#'
#' @examples
#' \donttest{
#' mydir <- system.file("permadir_examples_and_tests/convert_all", package = "scrutr")
#' outdir <- tempdir()
#'
#' convert_all(input_folderpath = mydir,
#'             considered_extensions = c("rds"),
#'             to = "csv",
#'             output_folderpath = outdir)
#'
#' list.files(outdir, pattern = "csv$")
#' }
convert_all <- function(input_folderpath, considered_extensions,
                        to, output_folderpath = input_folderpath) {

  if (!dir.exists(input_folderpath)) stop("input_folderpath must be a valid path")
  if (!is.character(considered_extensions)) stop("considered_extensions must be a character vector")
  if (!is.character(to) || length(to) != 1) stop("to must be a 1L character vector")
  if (!dir.exists(output_folderpath)) stop("output_folderpath must be a valid path")

  ext <- paste0(paste0("\\.", considered_extensions, "$"), collapse = "|")

  lfiles <- list.files(input_folderpath, full.names = TRUE)
  lfiles <- lfiles[grepl(ext, lfiles)]

  if (length(lfiles) == 0) {
    message("No files found matching the specified extensions.")
    return(invisible(character(0)))
  }

  out_paths <- character(length(lfiles))

  errors <- character(0)

  for (i in seq_along(lfiles)) {
    filepath <- lfiles[i]
    message("Converting: ", basename(filepath))

    tryCatch({
      data <- rio::import(filepath, trust = TRUE)
      data <- dplyr::mutate(data, dplyr::across(
        dplyr::everything(),
        \(y) { y[nchar(as.character(y)) == 0] <- NA; y }
      ))
      data <- dplyr::mutate(data, dplyr::across(
        dplyr::where(is.character),
        stringr::str_trim
      ))

      outname <- gsub(ext, paste0(".", to), basename(filepath))
      out_file <- file.path(output_folderpath, outname)
      rio::export(data, out_file)
      out_paths[i] <- out_file
      message("  Done: ", outname)
    }, error = function(e) {
      errors <<- c(errors, basename(filepath))
      warning("  Failed: ", basename(filepath), " - ", conditionMessage(e),
              call. = FALSE)
    })
  }

  if (length(errors) > 0) {
    warning(length(errors), " file(s) failed: ",
            paste(errors, collapse = ", "), call. = FALSE)
  }

  invisible(out_paths)
}

test_that("convert_all converts files correctly", {
  mydir <- system.file("permadir_examples_and_tests/convert_all", package = "scrutr")
  outdir <- file.path(tempdir(), "test_convert_all")
  dir.create(outdir, showWarnings = FALSE)
  on.exit(unlink(outdir, recursive = TRUE))

  result <- convert_all(
    input_folderpath = mydir,
    considered_extensions = c("rds", "csv"),
    to = "csv",
    output_folderpath = outdir
  )

  csv_files <- list.files(outdir, pattern = "\\.csv$")
  expect_true(length(csv_files) > 0)
})

test_that("convert_all handles no matching files", {
  outdir <- file.path(tempdir(), "test_convert_all_empty")
  dir.create(outdir, showWarnings = FALSE)
  on.exit(unlink(outdir, recursive = TRUE))

  expect_message(
    convert_all(
      input_folderpath = outdir,
      considered_extensions = "xyz",
      to = "csv"
    ),
    "No files found"
  )
})

test_that("convert_all errors on invalid input", {
  expect_error(convert_all(input_folderpath = "noexist"),
               "input_folderpath must be a valid path")

  mydir <- system.file("permadir_examples_and_tests/convert_all", package = "scrutr")

  expect_error(convert_all(input_folderpath = mydir, considered_extensions = 1),
               "considered_extensions must be a character vector")

  expect_error(convert_all(input_folderpath = mydir,
                           considered_extensions = "rds", to = 1),
               "to must be a 1L character vector")

  expect_error(convert_all(input_folderpath = mydir,
                           considered_extensions = "rds",
                           to = "csv", output_folderpath = "noexist"),
               "output_folderpath must be a valid path")
})

#' Inspect a data frame and write the output to an Excel file
#'
#' @param data_frame A data frame to inspect.
#' @param output_path Character. Folder path where the Excel output will be written.
#' @param output_label Character. Optional label for the output file.
#'   If \code{NULL}, defaults to the deparsed name of \code{data_frame}.
#' @return Invisibly returns the path to the written Excel file.
#'   The file contains the inspection table with prepended observation and variable counts.
#' @importFrom magrittr %>%
#' @export
#'
#' @examples
#' mydir <- file.path(tempdir(), "inspect_write_example")
#' dir.create(mydir)
#' inspect_write(data_frame = cars,
#'               output_path = mydir,
#'               output_label = "cars")
#' readxl::read_xlsx(file.path(mydir, "inspect_cars.xlsx"))
#' unlink(mydir, recursive = TRUE)
inspect_write <- function(data_frame, output_path, output_label = NULL) {

  if (!is.data.frame(data_frame)) {
    stop("data_frame must be a data.frame")
  }

  if (!dir.exists(output_path)) {
    stop("output_path doesn't exist. Check validity")
  }

  if (is.null(output_label)) {
    name <- deparse(substitute(data_frame))
  } else {
    name <- output_label
  }

  i <- inspect(data_frame)

  out_path <- file.path(output_path, paste0("inspect_", name, ".xlsx"))

  writexl::write_xlsx(
    rbind(
      c("Obs = ", nrow(data_frame), rep("", ncol(i) - 1)),
      c("Nvars = ", nrow(i), rep("", ncol(i) - 1)),
      cbind(1:nrow(i), i)
    ),
    out_path
  )

  invisible(out_path)
}

test_that("inspect_write writes correct Excel output", {
  mydir <- file.path(tempdir(), "test_inspect_write")
  dir.create(mydir, showWarnings = FALSE)
  on.exit(unlink(mydir, recursive = TRUE))

  result_path <- inspect_write(
    data_frame = cars,
    output_path = mydir,
    output_label = "cars"
  )

  expect_true(file.exists(file.path(mydir, "inspect_cars.xlsx")))
  expect_equal(result_path, file.path(mydir, "inspect_cars.xlsx"))

  content <- readxl::read_xlsx(file.path(mydir, "inspect_cars.xlsx"))

  # First row should be "Obs = " with value
  expect_equal(content[[1]][1], "Obs =")
  expect_equal(content[[2]][1], "50")

  # Second row should be "Nvars = "
  expect_equal(content[[1]][2], "Nvars =")
  expect_equal(content[[2]][2], "2")
})

test_that("inspect_write uses deparse(substitute) when no label", {
  mydir <- file.path(tempdir(), "test_inspect_write_nolabel")
  dir.create(mydir, showWarnings = FALSE)
  on.exit(unlink(mydir, recursive = TRUE))

  inspect_write(data_frame = cars, output_path = mydir)

  expect_true(file.exists(file.path(mydir, "inspect_cars.xlsx")))
})

test_that("inspect_write errors correctly", {
  mydir <- file.path(tempdir(), "test_inspect_write_err")
  dir.create(mydir, showWarnings = FALSE)
  on.exit(unlink(mydir, recursive = TRUE))

  expect_error(inspect_write(data_frame = "not_a_df", output_path = mydir),
               "data_frame must be a data.frame")

  expect_error(inspect_write(data_frame = cars, output_path = "/nonexistent/path"),
               "output_path doesn't exist")
})

---
title: "Getting started with scrutr"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting started with scrutr}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

`scrutr` helps you inspect, profile, and convert **collections of structured datasets**. This vignette walks through the main workflows.

## Setup

```{r setup}
library(scrutr)
```

## Inspecting a single data frame

`inspect()` produces a one-row-per-variable summary: class, distinct count, missing values, void strings, character lengths, and sample modalities.

```{r}
result <- inspect(CO2)
result
```

Use `nrow = TRUE` to also print the number of observations:

```{r}
result <- inspect(CO2, nrow = TRUE)
```

## Comparing variables across datasets

When working with several related tables, you often need to know which variables appear where and whether their types are consistent.

```{r}
data_list <- list(
  cars = cars,
  mtcars = mtcars[, c("mpg", "hp", "wt", "speed")  |> intersect(names(mtcars))],
  iris = iris
)

# Which variables are in which datasets?
vars_detect(data_list)
```

`vars_compclasses()` goes further and compares the class of each shared variable:

```{r}
# Use two datasets that share some columns
shared_list <- list(
  df1 = data.frame(x = 1:3, y = c("a", "b", "c"), stringsAsFactors = FALSE),
  df2 = data.frame(x = c(1.1, 2.2, 3.3), y = c("d", "e", "f"), stringsAsFactors = FALSE)
)

vars_compclasses(shared_list)
```

## Inspecting a whole folder of datasets

`inspect_vars()` is the main collection-level function. Point it at a folder, and it reads all matching files, inspects each one, then writes a comprehensive Excel report.

```{r, eval = FALSE}
# Create a temporary folder with example datasets
mydir <- file.path(tempdir(), "scrutr_demo")
dir.create(mydir, showWarnings = FALSE)

saveRDS(cars, file.path(mydir, "cars.rds"))
saveRDS(mtcars, file.path(mydir, "mtcars.rds"))
saveRDS(iris, file.path(mydir, "iris.rds"))

# Run the full inspection pipeline
inspect_vars(
  input_path = mydir,
  output_path = mydir,
  output_label = "demo",
  considered_extensions = "rds"
)

# The output Excel file contains multiple sheets:
# dims, inspect_tot, one sheet per dataset, vars_detect, vars_compclasses, etc.
list.files(mydir, pattern = "\\.xlsx$")
```

## Batch format conversion

### Simple folder conversion with `convert_all()`

Convert all matching files in a folder to another format:

```{r, eval = FALSE}
convert_all(
  input_folderpath = mydir,
  considered_extensions = "rds",
  to = "csv",
  output_folderpath = file.path(mydir, "csv")
)
```

### Mask-driven conversion with `convert_r()`

For more control, use an Excel mask that specifies exactly which files to convert and how to name the outputs:

```{r, eval = FALSE}
# 1. Generate a mask template
mask_convert_r(output_path = mydir)

# 2. Edit the mask in Excel, then:
convert_r(
  mask_filepath = file.path(mydir, "mask_convert_r.xlsx"),
  output_path = mydir
)
```

## Data hygiene helpers

`scrutr` includes several utilities for common data quality checks:

```{r}
# Find duplicates in a data frame
df <- data.frame(id = c(1, 2, 2, 3, 3, 3), value = letters[1:6])
dupl_show(df, "id")
```

```{r}
# Check a left join for key issues
left_df <- data.frame(key = c("a", "b", "c"))
right_df <- data.frame(key = c("a", "b", "b", "d"), val = 1:4)
ljoin_checks(left_df, right_df, "key")
```

## Path utilities

```{r}
paths <- c("data/raw/2024/file1.csv", "data/raw/2024/file2.csv")

# Keep only the first 2 levels
path_move(paths, "/", 2)

# Remove the last level (filename)
path_move(paths, "/", -1)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ljoin_checks.R
\name{ljoin_checks}
\alias{ljoin_checks}
\title{Perform a classical dplyr::left_join() and add check information related to join}
\usage{
ljoin_checks(ltable, rtable, ...)
}
\arguments{
\item{ltable}{Data.frame. Left data frame in the join}

\item{rtable}{Data.frame. Right data frame in the join}

\item{...}{Any other arguments passed to \code{dplyr::left_join()}.}
}
\value{
Data.frame. Output of dplyr::left_join() with messages on number of observations
in left, right and joined data frames and list of common variables between
ltable and rtable
}
\description{
Perform a classical dplyr::left_join() and add check information related to join
}
\examples{
left_table <- data.frame("person_id" = c(1, 1, 2, 3,
                                 2, 4, 5, 5 ,1),
                 "person_age" = c(25, 25, 21, 32,
                                  21, 48, 50, 50, 52),
                 "survey_month" = c("jan", "feb", "mar", "apr",
                                    "apr", "may", "jun", "jul", "jan"),
                 "survey_answer" = c("no", "yes", "no", "yes",
                                     "yes", "yes", "no", "yes", NA))

right_table <- data.frame("person_id" = c(2, 5, 4, 3, 1), 
                          "person_name" = c("John", "Marie", "Pierre", "Marc", "Jimmy"))

list("left_table" = left_table, 
     "right_table" = right_table)

ljoin_checks(left_table, right_table, by = "person_id")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mask_rename_r.R
\name{mask_rename_r}
\alias{mask_rename_r}
\title{Create an excel mask compatible with the rename_r() function.}
\usage{
mask_rename_r(input_path, output_filename = "mask_rename_r.xlsx")
}
\arguments{
\item{input_path}{Character 1L. Folder containing the set of files to rename}

\item{output_filename}{Character 1L. File name of the excel mask.}
}
\value{
No return value, called for side effects. An Excel mask file
is written to \code{input_path}.
}
\description{
Create an excel mask compatible with the rename_r() function. This must be used on a collection of files, i.e stored within the same folder.
}
\examples{
library(magrittr)
data(cars)
data(mtcars)

mydir <- tempfile()
dir.create(mydir)

saveRDS(cars, file.path(mydir, "cars.rds"))
saveRDS(mtcars, file.path(mydir, "mtcars.rds"))

list.files(mydir)

mask_rename_r(input_path = mydir)

list.files(mydir)

readxl::read_xlsx(file.path(mydir, "mask_rename_r.xlsx"))

unlink(mydir, recursive = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table_prop.R
\name{table_prop}
\alias{table_prop}
\title{Frequencies and proportions in one output}
\usage{
table_prop(..., margin = NULL, round = 3, noquote = FALSE)
}
\arguments{
\item{...}{Arguments passed to \code{base::table()}.}

\item{margin}{Integer 1L. The same argument as in base::prop.table()}

\item{round}{Integer 1L. Number of digits after decimal in base::prop.table() output}

\item{noquote}{Logical 1L. If TRUE, return an object of class noquote that provides better view of the output}
}
\value{
Matrix or noquote matrix. Frequencies with proportions in brackets, within a matrix
}
\description{
Combines base::table() and base::prop.table() outputs in a single one
}
\examples{
df <- data.frame(
                 "variable_1" = c("v1_1", "v1_1",
                                  "v1_2", "v1_2", "v1_2", "v1_2"),
                 "variable_2" = c("v2_1", "v2_1", "v2_1", "v2_1",
                                  "v2_2", "v2_2")
                                  )
table_prop(df$variable_1)
table_prop(df$variable_1, df$variable_2)
table_prop(df$variable_1, df$variable_2, margin = 2, noquote = TRUE)
df <- data.frame("person_id" = c(1, 1, 2, 3,
                                 2, 4, 5, 5 ,1),
                 "person_age" = c(25, 25, 21, 32,
                                  21, 48, 50, 50, 52),
                 "survey_month" = c("jan", "feb", "mar", "apr",
                                    "apr", "may", "jun", "jul", "jan"),
                 "survey_answer" = c("no", "yes", "no", "yes",
                                     "yes", "yes", "no", "yes", NA))

table_prop(df$survey_month)
table_prop(df$survey_month, df$survey_answer)
table_prop(df$survey_month, df$survey_answer, 
           margin = 2, round = 4)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inspect_write.R
\name{inspect_write}
\alias{inspect_write}
\title{Inspect a data frame and write the output to an Excel file}
\usage{
inspect_write(data_frame, output_path, output_label = NULL)
}
\arguments{
\item{data_frame}{A data frame to inspect.}

\item{output_path}{Character. Folder path where the Excel output will be written.}

\item{output_label}{Character. Optional label for the output file.
If \code{NULL}, defaults to the deparsed name of \code{data_frame}.}
}
\value{
Invisibly returns the path to the written Excel file.
The file contains the inspection table with prepended observation and variable counts.
}
\description{
Inspect a data frame and write the output to an Excel file
}
\examples{
mydir <- file.path(tempdir(), "inspect_write_example")
dir.create(mydir)
inspect_write(data_frame = cars,
              output_path = mydir,
              output_label = "cars")
readxl::read_xlsx(file.path(mydir, "inspect_cars.xlsx"))
unlink(mydir, recursive = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mask_convert_r.R
\name{mask_convert_r}
\alias{mask_convert_r}
\title{Create an excel mask compatible with the convert_r() function}
\usage{
mask_convert_r(output_path, output_filename = "mask_convert_r.xlsx")
}
\arguments{
\item{output_path}{Character 1L. Folder path where the mask will be created}

\item{output_filename}{Character 1L. File name (with extension) of the mask}
}
\value{
No return value, called for side effects. An Excel file
(\code{.xlsx}) is written to \code{file.path(output_path, output_filename)}
containing an empty template with columns \code{folder_path}, \code{file},
\code{converted_file}, and \code{to_convert}.
}
\description{
Create an excel mask compatible with the convert_r() function
}
\examples{
mydir <- file.path(tempdir(), "convert_r_tests_examples")
dir.create(mydir)

mask_convert_r(output_path = mydir)

list.files(mydir)

unlink(mydir, recursive = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inspect.R
\name{inspect}
\alias{inspect}
\title{Inspect a data frame}
\usage{
inspect(data_frame, nrow = FALSE)
}
\arguments{
\item{data_frame}{A data frame to inspect.}

\item{nrow}{Logical. If \code{TRUE}, the number of observations is printed
as a message in addition to the returned table.}
}
\value{
A tibble with one row per variable and columns: \code{variables},
\code{class}, \code{nb_distinct}, \code{prop_distinct}, \code{nb_na},
\code{prop_na}, \code{nb_void}, \code{prop_void}, \code{nchars}, \code{modalities}.
}
\description{
Compute a summary of each variable in a data frame: class, distinct values,
missing values, void strings, character lengths, and sample modalities.
}
\examples{
inspect(CO2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scrutr-package.R
\docType{package}
\name{scrutr-package}
\alias{scrutr}
\alias{scrutr-package}
\title{scrutr: Scrutinizing Collections of Structured Datasets}
\description{
Provides a coherent interface for exploring and transforming multiple related data frames that share a common structure. Complements single-dataset inspection tools by operating across an entire collection at once. Also includes lightweight utilities for related file and folder management tasks.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/danielrak/scrutr}
  \item Report bugs at \url{https://github.com/danielrak/scrutr/issues}
}

}
\author{
\strong{Maintainer}: Daniel Rakotomalala \email{rakdanielh@gmail.com}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/folder_structure_replicate.R
\name{folder_structure_replicate}
\alias{folder_structure_replicate}
\title{Replicate the folder structure of a given directory}
\usage{
folder_structure_replicate(dir, to)
}
\arguments{
\item{dir}{Character 1L. Path of directory which structure will be replicated}

\item{to}{Character 1L. Path of an output directory in which replicated structured will be placed}
}
\value{
No return value, called for side effects. The directory
structure of \code{dir} is recreated inside the \code{to} directory.
}
\description{
Replicate the folder structure of a given directory
}
\examples{
library(magrittr)

temp_dir_to_replicate <- tempfile()
dir.create(temp_dir_to_replicate)

dir.create(file.path(temp_dir_to_replicate, "dir1"))
dir.create(file.path(temp_dir_to_replicate, "dir2"))

temp_dir_out <- tempfile()
dir.create(temp_dir_out)

folder_structure_replicate(
  dir = temp_dir_to_replicate, 
  to = temp_dir_out)

unlink(temp_dir_to_replicate)
unlink(temp_dir_out)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dupl_sources.R
\name{dupl_sources}
\alias{dupl_sources}
\title{Illustrate sources of all duplicated values of a variable or a combination of variables}
\usage{
dupl_sources(data_frame, vars, output_as_df = FALSE)
}
\arguments{
\item{data_frame}{Data.frame. Input data frame. Must be in the Global Environment and has a data.frame class}

\item{vars}{Character. Vector of variable or combination of variables
from which duplicates are checked}

\item{output_as_df}{Logical 1L. If TRUE, output is rendered as a data.frame.}
}
\value{
List or data.frame. For each duplicated row regarding to vars, different values of the same variable
are shown, separated by AND
}
\description{
Illustrate sources of all duplicated values of a variable or a combination of variables
}
\examples{
# A fictional data with duplicated values:
df <- data.frame("person_id" = c(1, 1, 2, 3,
                                 2, 4, 5, 5 ,1),
                 "person_age" = c(25, 25, 21, 32,
                                  21, 48, 50, 50, 52),
                 "survey_month" = c("jan", "feb", "mar", "apr",
                                    "apr", "may", "jun", "jul", "jan"),
                 "survey_answer" = c("no", "yes", "no", "yes",
                                     "yes", "yes", "no", "yes", NA))

# Shuffling observations and columns to make duplicates difficult to see:
set.seed(1)
df <- df[sample(1:nrow(df)),
         sample(1:ncol(df))]
df

dupl_sources(data_frame = df, vars = "person_id")
dupl_sources(data_frame = df, vars = "person_id", output_as_df = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vars_compclasses.R
\name{vars_compclasses}
\alias{vars_compclasses}
\title{Collection-level variables types comparison}
\usage{
vars_compclasses(data_frames)
}
\arguments{
\item{data_frames}{A named list of data frames to compare.}
}
\value{
A data frame with a \code{vars_union} column and one column per dataset
showing the class of each variable (\code{"-"} if absent from that dataset).
}
\description{
Compare the class of each variable across a collection of datasets.
}
\examples{
data_list <- list(cars = cars, mtcars = mtcars)
vars_compclasses(data_list)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vars_detect.R
\name{vars_detect}
\alias{vars_detect}
\title{Variable detection patterns}
\usage{
vars_detect(data_frames)
}
\arguments{
\item{data_frames}{A named list of data frames to compare.}
}
\value{
A data frame with a \code{vars_union} column listing all variables,
and one column per dataset indicating \code{"ok"} (present) or \code{"-"} (absent).
Rows are sorted to highlight presence/absence patterns.
}
\description{
Detect the presence or absence of each variable across a collection of datasets.
}
\examples{
data_list <- list(cars = cars, mtcars = mtcars)
vars_detect(data_list)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vars_detect_not_everywhere.R
\name{vars_detect_not_everywhere}
\alias{vars_detect_not_everywhere}
\title{Variable detection - inconsistent patterns}
\usage{
vars_detect_not_everywhere(vars_detect_table)
}
\arguments{
\item{vars_detect_table}{Output of the \code{vars_detect()} function.}
}
\value{
A subset of \code{vars_detect_table} containing only variables
that are not present in every dataset.
}
\description{
Variable detection - inconsistent patterns
}
\examples{
data_list <- list(cars = cars, mtcars = mtcars)
vdetect_table <- vars_detect(data_list)
vars_detect_not_everywhere(vdetect_table)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert_r.R
\name{convert_r}
\alias{convert_r}
\title{Batch dataset format conversion using an Excel mask}
\usage{
convert_r(mask_filepath, output_path)
}
\arguments{
\item{mask_filepath}{Character. Full file path to the Excel mask.
The mask must contain columns: \code{folder_path}, \code{file},
\code{converted_file}, \code{to_convert} (1 to convert, 0 to skip).}

\item{output_path}{Character. Folder path where converted files will be placed.}
}
\value{
Invisibly returns a character vector of output file paths.
}
\description{
Read an Excel mask describing files to convert, then import, clean, and
re-export each file in the desired format. Cleaning includes replacing void
character values (\code{""}) with \code{NA} and trimming whitespace.
}
\examples{
mydir <- system.file("permadir_examples_and_tests/convert_r", package = "scrutr")

mask <- data.frame(
  folder_path = rep(mydir, 2),
  file = c("original_cars.rds", "original_mtcars.csv"),
  converted_file = c("converted_cars.csv", "converted_mtcars.csv"),
  to_convert = rep(1, 2)
)

\donttest{
mask_path <- file.path(tempdir(), "mask_convert_r.xlsx")
writexl::write_xlsx(mask, mask_path)

convert_r(
  mask_filepath = mask_path,
  output_path = tempdir()
)

# Clean up:
file.remove(file.path(tempdir(),
  c("converted_cars.csv", "converted_mtcars.csv", "mask_convert_r.xlsx")))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dupl_show.R
\name{dupl_show}
\alias{dupl_show}
\title{Show observations of all duplicated values of a variable or a combination of variables}
\usage{
dupl_show(data_frame, vars)
}
\arguments{
\item{data_frame}{Data.frame. Input data frame. Must be in the Global Environment and has a data.frame class}

\item{vars}{Character. Vector of variable or combination of variables
from which duplicates are checked}
}
\value{
Data.frame. The part of inputted data frame with all observations of duplicated values of
indicated variable or combination of variables
}
\description{
Show observations of all duplicated values of a variable or a combination of variables
}
\examples{
# A fictional data with duplicated values:
df <- data.frame("person_id" = c(1, 1, 2, 3,
                                 2, 4, 5, 5 ,1),
                 "person_age" = c(25, 25, 21, 32,
                                  21, 48, 50, 50, 52),
                 "survey_month" = c("jan", "feb", "mar", "apr",
                                    "apr", "may", "jun", "jul", "jan"),
                 "survey_answer" = c("no", "yes", "no", "yes",
                                     "yes", "yes", "no", "yes", NA))

# Shuffling observations and columns to make duplicates difficult to see:
set.seed(1)
df <- df[sample(1:nrow(df)),
         sample(1:ncol(df))]
df

# See all of the rows where person_id has more than an unique possible value: 
dupl_show(data = df, var = "person_id")

# See all of the rows where the combination of person_id and survey_month variables has 
# more than an unique possible value : 
dupl_show(data = df, var = c("person_id", "survey_month"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/detect_chars_structure.R
\name{detect_chars_structure}
\alias{detect_chars_structure}
\title{Detect character structure
Detect if values within a character variable match at least one of defined patterns}
\usage{
detect_chars_structure(vector, patterns, verbose = FALSE)
}
\arguments{
\item{vector}{Character. Input vector to detect pattern from}

\item{patterns}{Character. Patterns to detect within vector. Regex is supported}

\item{verbose}{Logical 1L. If TRUE, additional details related to the pattern detection are provided}
}
\value{
Logical 1L. If verbose is set to TRUE, the function returns a list with the following elements in order:
\itemize{
\item "Any defined structure" : Logical 1L. TRUE if the pattern is detected anywhere from the input vector
\item "Which" : Character. Unique values of input vector matching the defined patterns
\item "Where" : Integer. Indexes of values from input vector matching the defined patterns
}
}
\description{
Detect character structure
Detect if values within a character variable match at least one of defined patterns
}
\examples{
detect_chars_structure(
  vector = c("ABCD1234", "4567EF", "89GHIJ10"), 
  patterns = "[:alpha:]{4}" # detect four consecutive alphabetic values
)

detect_chars_structure(
  vector = c("ABCD1234", "4567EF", "89GHIJ10"), 
  patterns = "[:alpha:]{4}", 
  verbose = TRUE
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rename_r.R
\name{rename_r}
\alias{rename_r}
\title{Industrialized file renaming}
\usage{
rename_r(mask_filepath)
}
\arguments{
\item{mask_filepath}{Character 1L. Entire file path of the excel mask}
}
\value{
No return value, called for side effects. Files are renamed
on disk according to the instructions in the Excel mask.
}
\description{
Industrialized file renaming
}
\examples{
library(magrittr)
data(cars)
data(mtcars)

mydir <- tempfile()
dir.create(mydir)

# Two example files to rename: 
saveRDS(cars, file.path(mydir, "cars.rds"))
saveRDS(mtcars, file.path(mydir, "mtcars.rds"))
list.files(mydir)

# Create the mask: 
mask_rename_r(input_path = mydir)

# Fill the mask (in practice you can do it manually): 
mask <- rio::import(file.path(mydir, "mask_rename_r.xlsx"))
mask[["renamed_file"]] <- c("cars_renamed.rds", "mtcars_renamed.rds")
mask[["to_rename"]] <- rep(1, 2)
writexl::write_xlsx(mask, file.path(mydir, "mask_rename_r.xlsx"))

# Apply the rename function: 
rename_r(mask_filepath = file.path(mydir, "mask_rename_r.xlsx"))

# See the renamed files: 
list.files(mydir)

# Clean tempdir: 
unlink(mydir, recursive = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chars_structure_general.R
\name{chars_structure_general}
\alias{chars_structure_general}
\title{Generalized characters structure}
\usage{
chars_structure_general(
  input_vector,
  split = "",
  patterns_and_replacements,
  unique = TRUE,
  named_output = TRUE
)
}
\arguments{
\item{input_vector}{Character. Vector to process}

\item{split}{Character 1L. Symbol separator}

\item{patterns_and_replacements}{Character. Named character vector of patterns (names) and replacements (values)}

\item{unique}{Logical 1L. If TRUE, the result is reduced to unique values}

\item{named_output}{Logical 1L. If TRUE, output vector is named after corresponding input values.}
}
\value{
A character vector describing the generalized structure of each
element of \code{input_vector}. If \code{named_output} is \code{TRUE},
the vector is named after the corresponding input values. If
\code{unique} is \code{TRUE}, only structures for unique input values
are returned; otherwise one structure per element of \code{input_vector}.
}
\description{
Get any user-defined patterns from each symbol of character vector
}
\examples{

input <- c("ABC123", "DE4F56", "789GHI", "ABC123")

# Default values of unique and named_output:
chars_structure_general(input_vector = input, split = "", 
                        patterns_and_replacements = c("[:alpha:]" = "[letter]", 
                                                      "[:digit:]" = "[number]"), 
                        unique = TRUE, named_output = TRUE)

# unique is set to default value TRUE and named_output is set to FALSE:
chars_structure_general(input_vector = input, split = "", 
                        patterns_and_replacements = c("[:alpha:]" = "[letter]", 
                                                      "[:digit:]" = "[number]"), 
                        unique = TRUE, named_output = FALSE)

# unique is set to FALSE and named_output to FALSE:
chars_structure_general(input_vector = input, split = "", 
                        patterns_and_replacements = c("[:alpha:]" = "[letter]", 
                                                      "[:digit:]" = "[number]"), 
                        unique = FALSE, named_output = FALSE)

# unique is set to FALSE and named_output to defalut value TRUE:
chars_structure_general(input_vector = input, split = "", 
                        patterns_and_replacements = c("[:alpha:]" = "[letter]", 
                                                      "[:digit:]" = "[number]"), 
                        unique = FALSE, named_output = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chars_structure.R
\name{chars_structure}
\alias{chars_structure}
\title{Get :alpha: / :digit: patterns from each symbol of character vector}
\usage{
chars_structure(input_vector, unique = TRUE, named_output = TRUE)
}
\arguments{
\item{input_vector}{Character. Vector to process}

\item{unique}{Logical 1L. If TRUE, the result is reduced to unique values.}

\item{named_output}{Logical 1L. If TRUE, output vector is named after corresponding input values.}
}
\value{
Character. Vector describing structure of each element of input_vector,
see example.
}
\description{
Get :alpha: / :digit: patterns from each symbol of character vector
}
\examples{
library(magrittr)
input <- c("ABC123", "DE4F56", "789GHI", "ABC123")

# Default values of unique and named_output: 
chars_structure(input_vector = input, unique = TRUE, named_output = TRUE)

# unique is set to default value TRUE and named_output is set to FALSE: 
chars_structure(input_vector = input, unique = TRUE, named_output = FALSE)

# unique is set to FALSE and named_output to FALSE: 
chars_structure(input_vector = input, unique = FALSE, named_output = FALSE)

# unique is set to FALSE and named_output to defalut value TRUE: 
chars_structure(input_vector = input, unique = FALSE, named_output = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vars_compclasses_not_allsame.R
\name{vars_compclasses_not_allsame}
\alias{vars_compclasses_not_allsame}
\title{Variable class comparison - inconsistent types across datasets}
\usage{
vars_compclasses_not_allsame(vars_compclasses_table)
}
\arguments{
\item{vars_compclasses_table}{Output of the \code{vars_compclasses()} function.}
}
\value{
A subset of \code{vars_compclasses_table} containing only variables
with different classes across datasets where they appear.
}
\description{
Variable class comparison - inconsistent types across datasets
}
\examples{
data_list <- list(cars = cars, mtcars = mtcars)
vcompclasses <- vars_compclasses(data_list)
vars_compclasses_not_allsame(vcompclasses)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/replace_multiple.R
\name{replace_multiple}
\alias{replace_multiple}
\title{Replace character vector values using a correspondence approach}
\usage{
replace_multiple(input_vector, replacements, replace_all = FALSE)
}
\arguments{
\item{input_vector}{Character. Character vector on which replacements take place.}

\item{replacements}{Character. Named character vector defining replacement
correspondences (names = patterns, values = replacements). Names are
treated as literal strings (regex metacharacters such as \code{.},
\code{(}, \code{+} are not interpreted).}

\item{replace_all}{Logical. If \code{TRUE}, \code{stringr::str_replace_all()}
is used instead of \code{stringr::str_replace()}.}
}
\value{
Character vector with replacements applied.
}
\description{
Names of \code{replacements} are matched literally (not as regular
expressions). Elements of \code{input_vector} that match none of the
patterns are returned unchanged.
}
\section{Overlapping patterns}{

When several patterns can match the same element, the first match found
in \code{input_vector} (scanned left to right) is used. When patterns
start at the same position, the one listed first in \code{replacements}
wins, not the longest. Order \code{replacements} accordingly if some
patterns are prefixes of others.
}

\examples{
input <- c("one-one", "two-two-one", "three-three-two")

replace_multiple(input,
                 replacements =
                   c("one" = "1", "two" = "2",
                     "three" = "3"))

replace_multiple(input,
                 replacements =
                   c("one" = "1", "two" = "2",
                     "three" = "3"),
                 replace_all = TRUE)

# Unmatched elements are returned as-is:
replace_multiple(c("one", "unmatched"), c("one" = "1"))

# Regex metacharacters are matched literally:
replace_multiple(c("a.b", "aXb"), c("." = "DOT"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/path_move.R
\name{path_move}
\alias{path_move}
\title{Move through paths}
\usage{
path_move(path_vector, path_separator = "/", move)
}
\arguments{
\item{path_vector}{Character. Vector of paths with equal number of levels.}

\item{path_separator}{Character. Path separator
(adapted to your OS, e.g. \code{"/"}).}

\item{move}{Integer. If positive, outputs path up to the specified level.
If negative, removes the last specified level(s).}
}
\value{
Character vector of transformed paths.
}
\description{
Move through paths
}
\examples{
pvector <- c(
  "level_1/level_2/level_3/file_1.ext",
  "level_1/level_2/level_3/file_2.ext"
)

path_move(path_vector = pvector,
          path_separator = "/",
          move = 1)

path_move(path_vector = pvector,
          path_separator = "/",
          move = 2)

path_move(path_vector = pvector,
          path_separator = "/",
          move = -1)

path_move(path_vector = pvector,
          path_separator = "/",
          move = -2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert_all.R
\name{convert_all}
\alias{convert_all}
\title{Convert all datasets within a folder to a given file format}
\usage{
convert_all(
  input_folderpath,
  considered_extensions,
  to,
  output_folderpath = input_folderpath
)
}
\arguments{
\item{input_folderpath}{Character. Folder path containing datasets to convert.}

\item{considered_extensions}{Character vector. File extensions to include
(must be supported by \pkg{rio}).}

\item{to}{Character. The target output format (must be supported by \pkg{rio}).}

\item{output_folderpath}{Character. Folder path for converted files
(defaults to \code{input_folderpath}).}
}
\value{
Invisibly returns a character vector of output file paths.
}
\description{
Scan a folder for files matching specified extensions, then convert each one
to the target format. Void character values are replaced with \code{NA} and
character whitespace is trimmed.
}
\examples{
\donttest{
mydir <- system.file("permadir_examples_and_tests/convert_all", package = "scrutr")
outdir <- tempdir()

convert_all(input_folderpath = mydir,
            considered_extensions = c("rds"),
            to = "csv",
            output_folderpath = outdir)

list.files(outdir, pattern = "csv$")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inspect_vars.R
\name{inspect_vars}
\alias{inspect_vars}
\title{Inspect a collection of datasets}
\usage{
inspect_vars(
  input_path,
  output_path,
  output_label,
  considered_extensions,
  encoding = NULL
)
}
\arguments{
\item{input_path}{Character. Folder path containing datasets to explore.}

\item{output_path}{Character. Folder path where the Excel output will be written.}

\item{output_label}{Character. A concise label for the output file name.}

\item{considered_extensions}{Character vector. File extensions to include
(without the dot, e.g. \code{"rds"} not \code{".rds"}).}

\item{encoding}{Character 1L or \code{NULL}. Encoding passed to
\code{rio::import()} for text-based formats (CSV, TSV). Must be one of
\code{"unknown"}, \code{"UTF-8"}, or \code{"Latin-1"}. Defaults to
\code{NULL} (let \code{rio} / \code{data.table} pick the default).
Ignored for binary formats (e.g. RDS, xlsx).}
}
\value{
Invisibly returns the path to the written Excel file.
The file contains sheets: dims, inspect_tot, one per dataset inspection,
vars_detect, vars_detect_everywhere, vars_detect_not_everywhere,
vars_compclasses, vars_compclasses_allsame, vars_compclasses_not_allsame.
}
\description{
Read all datasets from a folder matching the specified extensions, inspect each one,
and write a comprehensive Excel report covering variable presence, types, and per-dataset summaries.
}
\examples{
mydir <- file.path(tempdir(), "inspect_vars_example")
dir.create(mydir)

saveRDS(cars, file.path(mydir, "cars1.rds"))
saveRDS(mtcars, file.path(mydir, "cars2.rds"))

inspect_vars(input_path = mydir, output_path = mydir,
             output_label = "cardata",
             considered_extensions = "rds")

# Read back the 10 sheets:
purrr::map(1:10, function(x)
           rio::import(file.path(mydir, "inspect_vars_cardata.xlsx"),
                       sheet = x))

unlink(mydir, recursive = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/detect_chars_structure_datasets.R
\name{detect_chars_structure_datasets}
\alias{detect_chars_structure_datasets}
\title{Detect character structure from datasets}
\usage{
detect_chars_structure_datasets(
  datasets_folderpath,
  considered_extensions,
  patterns,
  output_filepath = file.path(datasets_folderpath, paste0("detect_chars_structure_",
    basename(datasets_folderpath), ".rds")),
  get_output_in_session = TRUE
)
}
\arguments{
\item{datasets_folderpath}{Character 1L. Folder path of datasets to process. These datasets must be at the root of the path}

\item{considered_extensions}{Character. Datasets file extensions to consider. Extensions must be one supported by the rio:: package}

\item{patterns}{Character. Patterns to detect across the datasets variables. Regex is supported}

\item{output_filepath}{Character 1L. Output folder path.}

\item{get_output_in_session}{Logical 1L. If TRUE, the function return a list, such that each element element corresponds to pattern detection details for each considered dataset}
}
\value{
If \code{get_output_in_session} is \code{TRUE}, a named list of data
frames (one per dataset file), each with columns \code{var} (variable name),
\code{any_defined_structure} (logical), and \code{examples} (character).
The list is also saved as an RDS file at \code{output_filepath}.
If \code{get_output_in_session} is \code{FALSE}, the function returns
\code{NULL} invisibly and is called for its side effect of writing the RDS file.
}
\description{
Detect character structure from datasets
}
\examples{
mydir <- system.file("detect_chars_structure_datasets", package = "scrutr")
outfile <- file.path(tempdir(), "detect_college.rds")

detect <- detect_chars_structure_datasets(
  datasets_folderpath = mydir,
  considered_extensions = "xlsx",
  patterns = "(?i)college",
  output_filepath = outfile,
  get_output_in_session = TRUE)

# head(lapply(detect, head))

file.exists(outfile)
unlink(outfile)
}
